import axios from 'axios'
import dataSetUtils from '../../../classes/dataSetUtils'
import constructors from '../../../classes/constructors'
import { uuid } from 'vue-uuid'
import Router from 'vue-router'

const storeUtils = {
  router: null,
  pRouter: Promise.resolve(),
}

export default {
  addLog: ({ commit }, payload) => {
    commit('ADD_LOG', payload)
  },
  setSessionId: ({ commit }, payload) => {
    commit('SET_SESSIONID', payload)
  },
  setClientRequest: ({ commit }, payload) => {
    commit('SET_CLIENT_REQUEST', payload)
  },
  setRouter(_, { router }) {
    storeUtils.router = router
  },
  checkRouter({ getters, dispatch }) {
    if (storeUtils.router instanceof Router) return true
    if (!getters.workAsPlugin)
      dispatch('setErrors', {
        errors:
          'Błąd techniczny: brak routera.\nSkontaktuj się z administratorem aplikacji.',
      })
    return false
  },
  setLanguageSuffix: ({ commit }, payload) => {
    commit('SET_LANGUAGESUFFIX', payload)
  },
  setLookupDefVisible: ({ commit, getters }, payload) => {
    const lookupDef = getters.getDataSetLookupDef(
      payload.appWindowIdent,
      payload.dataSetIdent,
      payload.dataFieldIdent
    )
    const lookupVisible = payload.lookupVisible
    const appWindow = getters.getAppWindow(lookupDef.appWindowIdentLookup)
    commit('SET_LOOKUP_DEF_VISIBLE', {
      lookupDef: lookupDef,
      lookupVisible: lookupVisible,
      dataSet: payload.dataSet,
      row: payload.row,
      appWindowUniqIdent: payload.appWindowIdent,
    })
    commit('SET_APPWINDOW_LOOKUP_DEF', {
      lookupDef,
      appWindow,
    })
  },
  setLookupDefSelection: async ({ dispatch, commit, getters }, payload) => {
    const lookupDef = getters.getAppWindowLookupDef(payload.appWindowIdent)
    if (lookupDef !== null) {
      const lookupVisible = false
      const { row, sourceKind } = lookupDef
      commit('SET_LOOKUP_DEF_VISIBLE', {
        lookupDef: lookupDef,
        lookupVisible: lookupVisible,
        dataSet: payload.dataSet,
        row: payload.row,
      })
      await dispatch('deleteDataSetWhereListsItem', {
        appWindowIdent: lookupDef.appWindowUniqIdent,
        dataSetIdent: lookupDef.dataSetIdent,
        dataFieldIdent: lookupDef.dataFieldIdent,
        sourceKind: sourceKind,
        row: row,
      })
      await dispatch('fillDataSetsFromSetters', {
        appWindowIdent: payload.appWindowIdent,
        lookupAppWindowIdent: lookupDef.appWindowUniqIdent,
        lookupDataSetIdent: lookupDef.dataSetIdent,
        lookupDataFieldIdent: lookupDef.dataFieldIdent,
        lookupIsMultiSelect: lookupDef.isMultiSelect,
        sourceKind: sourceKind,
        row: row,
      })
    }
  },
  setErrors: ({ getters, commit, dispatch, state }, payload) => {
    if (!payload) return
    let errors = payload.errors
    const row = payload.row,
      appWindowIdent = payload.appWindowIdent,
      dataSetIdent = payload.dataSetIdent,
      id = payload.id,
      action = payload.action,
      oprIdent = payload.oprIdent
    //
    //Filter is used here to implicitly create a copy of the array.
    //If forEach was checking the condition, the array being iterated
    // could be mutated during the iteration.
    state.errors
      .filter(
        error =>
          error.appWindowIdent == appWindowIdent &&
          error.dataSetIdent == dataSetIdent &&
          error.oprIdent == oprIdent &&
          (error.row == row || error.row == null && row == null)
      )
      .forEach(error => commit('DEL_ERROR', { error: error }))
    //
    if (!errors) return
    if (!Array.isArray(errors)) {
      if (errors.code && errors.originalError && errors.originalError.info) {
        errors = [{ messageText: errors.originalError.info.message }]
      } else {
        errors = [{ messageText: errors }]
      }
    }
    errors.reverse().forEach(error => {
      error = {
        ...error,
        appWindowIdent: appWindowIdent,
        dataSetIdent: dataSetIdent,
        row: row,
        oprIdent: oprIdent,
        __uuid: id,
        isVisible: true,
      }
      const autoCloseTimeout = error.autoCloseTimeout
      const showKind =
        error.showKind === null || error.showKind === undefined ? 0 : error.showKind
      if (appWindowIdent && dataSetIdent && error.errorFields) {
        const messageType =
          error.messageType === null || error.messageType === undefined
            ? 1
            : error.messageType
        error.errorFields.forEach(errorField => {
          const fieldsType = getters.getFieldsType(messageType)
          let autoCloseTimeoutId
          if (autoCloseTimeout) {
            autoCloseTimeoutId = setTimeout(
              () =>
                dispatch('removeError', {
                  error: error,
                }),
              autoCloseTimeout
            )
          }
          if (row) {
            const rowMessageField = getters.getRowMessageFields(
              row,
              fieldsType,
              errorField.errorField
            )
            if (
              rowMessageField != null &&
              rowMessageField.autoCloseTimeoutIdToClear != null
            ) {
              commit('MESSAGE_FIELD_CLEAR_TIMEOUT', {
                messageField: rowMessageField,
              })
            }
            commit('ADD_ROW_MESSAGE_FIELD', {
              row: row,
              fieldsType: fieldsType,
              messageField: errorField.errorField,
              message: {
                messageText: error.messageText,
                messageType: messageType,
                showKind: showKind,
                autoCloseTimeoutId: autoCloseTimeoutId,
              },
            })
          } else {
            const dataSet = getters.getDataSet(appWindowIdent, dataSetIdent)
            const dataSetMessageField = getters.getDataSetMessageField(
              dataSet,
              fieldsType,
              errorField.errorField
            )
            if (
              dataSetMessageField != null &&
              dataSetMessageField.autoCloseTimeoutIdToClear != null
            ) {
              commit('MESSAGE_FIELD_CLEAR_TIMEOUT', {
                messageField: dataSetMessageField,
              })
            }
            commit('ADD_DATASET_MESSAGE_FIELD', {
              dataSet: dataSet,
              fieldsType: fieldsType,
              messageField: errorField.errorField,
              message: {
                messageText: error.messageText,
                messageType: messageType,
                showKind: showKind,
                autoCloseTimeoutId: autoCloseTimeoutId,
              },
            })
          }
        })
      }
      if (showKind == 0) {
        commit('ADD_ERROR', { error })
      }
    })
    commit('SET_DATASET_ACTION_STATE', { row: row, action: action, state: -1 })
  },
  fillDataSetWhere: async ({ dispatch, getters }, payload) => {
    let appWindowIdentFrom
    let dataSetIdentFrom
    let dataSetsFrom
    let dataSetsFromItem
    let dataSetFrom
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    let whereList
    if (dataSet) {
      dataSetsFrom = dataSet.dataSetsFrom
    }
    if (dataSetsFrom != null) {
      for (let i = 0; i < dataSetsFrom.length; i++) {
        dataSetsFromItem = dataSetsFrom[i]
        appWindowIdentFrom = dataSetsFromItem.appWindowIdentFrom
        dataSetIdentFrom = dataSetsFromItem.dataSetIdentFrom
        dataSetFrom = getters.getDataSet(appWindowIdentFrom, dataSetIdentFrom)
        whereList = dataSetFrom.whereLists
          ? dataSetFrom.whereLists[dataSetsFromItem.dataFieldIdentFrom]
          : null
        await dispatch('setDataSetFieldValue', {
          sourceKind: dataSetsFromItem.sourceKindTo,
          dataSet: dataSet,
          dataFieldIdent: dataSetsFromItem.dataFieldIdentTo,
          dataFieldValue: getters.getDataSetFieldValue(
            dataSetsFromItem.sourceKindFrom,
            dataSetFrom,
            dataSetsFromItem.dataFieldIdentFrom
          ),
        })
        if (dataSetsFromItem.sourceKindTo === 'where') {
          dispatch('setDataSetWhereListsFieldValue', {
            dataSet: dataSet,
            lookupDataFieldIdent: dataSetsFromItem.dataFieldIdentTo,
            whereListValue: null,
          })
          if (whereList) {
            await whereList.forEach(whereListItem => {
              dispatch('setDataSetWhereListsFieldValue', {
                dataSet: dataSet,
                lookupDataFieldIdent: dataSetsFromItem.dataFieldIdentTo,
                whereListValue: whereListItem,
              })
            })
          }
        }
      }
    }
    dispatch('updateUrlQuery', { dataSet })
  },
  fillDataSetsFromGetters: async ({ dispatch, getters }, payload) => {
    const lookupGetters = getters.getAppWindowLookupGetters(payload.appWindowIdent)
    const lookupDef = getters.getAppWindowLookupDef(payload.appWindowIdent)
    if (lookupDef == null) return
    const row = payload.row
    await lookupGetters.forEach(async lookupGetter => {
      if (
        payload.dataSetIdent === null ||
        payload.dataSetIdent === undefined ||
        lookupGetter.dataSetIdentTo === payload.dataSetIdent
      ) {
        const dataSetTo = getters.getDataSet(
          lookupGetter.appWindowIdentTo,
          lookupGetter.dataSetIdentTo
        )
        const dataSetFrom = getters.getDataSet(
          lookupGetter.appWindowIdentFrom === lookupDef.appWindowIdent
            ? lookupDef.appWindowUniqIdent
            : lookupGetter.appWindowIdentFrom,
          lookupGetter.dataSetIdentFrom
        )
        let value
        if (lookupGetter.sourceKindFrom === 'value') {
          value = lookupGetter.dataFieldValueFrom
        } else if (
          row != null &&
          row._appWindowIdent === lookupGetter.appWindowIdentFrom &&
          row._dataSetIdent === lookupGetter.dataSetIdentFrom &&
          row._sourceKind === lookupGetter.sourceKindFrom
        )
          value = row[lookupGetter.dataFieldIdentFrom]
        else
          value = getters.getDataSetFieldValue(
            lookupGetter.sourceKindFrom,
            dataSetFrom,
            lookupGetter.dataFieldIdentFrom
          )

        await dispatch('setDataSetFieldValue', {
          sourceKind: lookupGetter.sourceKindTo,
          dataSet: dataSetTo,
          dataFieldIdent: lookupGetter.dataFieldIdentTo,
          dataFieldValue: value,
        })
        if (
          lookupGetter.sourceKindFrom === 'where' &&
          lookupGetter.sourceKindTo === 'where'
        ) {
          const whereListValue =
            dataSetFrom.whereLists[lookupGetter.dataFieldIdentFrom] || null
          if (whereListValue)
            await whereListValue.forEach(async whereListItem => {
              await dispatch('setDataSetWhereListsFieldValue', {
                dataSet: dataSetTo,
                lookupDataFieldIdent: lookupGetter.dataFieldIdentTo,
                whereListValue: whereListItem,
              })
            })
          else {
            await dispatch('setDataSetWhereListsFieldValue', {
              dataSet: dataSetTo,
              lookupDataFieldIdent: lookupGetter.dataFieldIdentTo,
              whereListValue: null,
            })
          }
        }
      }
    })
  },
  refreshDataOnFromQuery: async ({ dispatch }, payload) => {
    if (!payload.query) return
    await dispatch('fillDataSetsFromQuery', {
      appWindowIdent: payload.appWindowIdent,
      query: payload.query,
    })
    await dispatch('setAppWindowData', {
      appWindowIdent: payload.appWindowIdent,
      serverSide: payload.serverSide,
      query: payload.query,
    })
  },
  fillDataSetsFromQuery: async ({ dispatch, getters }, payload) => {
    const appWindowIdent = payload.appWindowIdent
    const appWindow = getters.getAppWindow(appWindowIdent)
    if (!appWindow) return

    const queryDataSetsTo = appWindow.queryDataSetsTo
    if (!queryDataSetsTo) return

    const { query } = payload

    await Promise.all(queryDataSetsTo.map(async item => {
      const dataSet = getters.getDataSet(appWindowIdent, item.dataSetIdent)
      const where = dataSet.where || {}
      const whereFieldsIdent = getters.getDataSetWhereFieldsIdentsDataField(dataSet, item.dataFieldIdent)
      const whereFieldsIdentValue = whereFieldsIdent?.dataFieldValueDef || null
      const newVal = query?.[item.linkedParamName] || whereFieldsIdentValue
      if (newVal !== where[item.dataFieldIdent]) {
        await dispatch('setDataSetWhereFieldValue', {
          appWindowIdent: appWindowIdent,
          dataSet: dataSet,
          dataFieldIdent: item.dataFieldIdent,
          dataFieldValue: newVal,
        })
      }
    }))

    if (appWindow.dataSets != null) {
      for (const dsi in appWindow.dataSets) {
        const ds = appWindow.dataSets[dsi]
        if (getters.getDataSetFieldsIdentsDataField(ds, '_urlParamName') == null) continue
        dispatch('deleteAllSelectedRow', {
          dataSetIdent: dsi,
          appWindowIdent: payload.appWindowIdent,
          urlLock: 1,
        })
        if (query == null) continue
        ds.rows.forEach(row => {
          switch (row._urlParamKind) {
            case 'list': {
              if (row._urlParamValue == null) break
              if (
                query[row._urlParamName] instanceof Array &&
                query[row._urlParamName].find(p => p == row._urlParamValue)
                ||
                query[row._urlParamName] == row._urlParamValue
              ) {
                dispatch('addSelectedRow', {
                  dataSetIdent: dsi,
                  appWindowIdent: payload.appWindowIdent,
                  row: row,
                  urlLock: 1,
                })
              }
            }
          }
        })
      }
    }
  },
  fillDataSetsFromData: async ({ dispatch, getters }, payload) => {
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    const data = payload.data
    const filter = payload.filter
    const row = filter
      ? getters.appWindowDataSetFindRow(
        payload.appWindowIdent,
        payload.dataSetIdent,
        filter
      )
      : payload.row
    const sourceKindTo = payload.sourceKindTo || 'where'
    const onCFLock = payload.onCFLock
    if (!data) return
    data.forEach(async item => {
      if (item.dataFieldIdent) {
        await dispatch(
          sourceKindTo === 'where'
            ? 'setDataSetWhereFieldValue'
            : 'setDataSetRowsFieldValue',
          {
            appWindowIdent: payload.appWindowIdent,
            dataSet: dataSet,
            row: row,
            dataFieldIdent: item.dataFieldIdent,
            dataFieldValue: item.dataFieldValue,
            onCFLock: onCFLock,
          }
        )
      }
    })
  },
  addRow: async ({ commit }, payload) => {
    const dataSet = payload.dataSet
    const appWindowIdent = payload.appWindowIdent
    const fieldsIdents = payload.fieldsIdents
    const row = payload.row
    commit('ADD_ROW', { appWindowIdent, dataSet, fieldsIdents, row })
  },
  deleteRow: async ({ commit }, payload) => {
    const dataSet = payload.dataSet
    const row = payload.row
    commit('DELETE_ROW', { dataSet, row })
  },
  addFile: async ({ commit }, payload) => {
    const dataSet = payload.dataSet
    const file = payload.file
    commit('ADD_FILE', { dataSet, file })
  },
  deleteFile: async ({ commit }, payload) => {
    const dataSet = payload.dataSet
    const file = payload.file
    commit('DELETE_FILE', { dataSet, file })
  },
  addSelectedRow: async ({ commit, getters, dispatch }, payload) => {
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    if (!dataSet.keyFieldsIdents || dataSet.keyFieldsIdents.length === 0) {
      await dispatch('setErrors', {
        appWindowIdent: payload.appWindowIdent,
        dataSetIdent: payload.dataSetIdent,
        errors:
          'Brak zdefiniowanych pól kluczowych dla źródła danych: ' + payload.dataSetIdent,
      })
      return -1
    }
    const row = payload.row
    commit('ADD_SELECTED_ROW', { dataSet, row })
    if (payload.urlLock !== 1) {
      dispatch('updateUrlQuery', { dataSet: dataSet, selectedRowsAdd: [row] })
      dispatch('refreshDataSetDependence', {
        appWindowIdent: payload.appWindowIdent,
        parentDataSetIdent: dataSet.dataSetIdent,
        refreshAggregates: 1,
        sourceKindFrom: 'selectedRows',
      })
    }
  },
  addAllSelectedRow: async ({ commit, getters, dispatch }, payload) => {
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    if (!dataSet.keyFieldsIdents || dataSet.keyFieldsIdents.length === 0) {
      await dispatch('setErrors', {
        appWindowIdent: payload.appWindowIdent,
        dataSetIdent: payload.dataSetIdent,
        errors:
          'Brak zdefiniowanych pól kluczowych dla źródła danych: ' + payload.dataSetIdent,
      })
      return -1
    }
    if (dataSet && dataSet.rows)
      await dataSet.rows.forEach(row => {
        commit('ADD_SELECTED_ROW', { dataSet, row })
      })
    dispatch('updateUrlQuery', {
      dataSet: dataSet,
      selectedRowsAdd: dataSet.selectedRows,
    })
    dispatch('refreshDataSetDependence', {
      appWindowIdent: payload.appWindowIdent,
      parentDataSetIdent: dataSet.dataSetIdent,
      refreshAggregates: 1,
      sourceKindFrom: 'selectedRows',
    })
  },
  deleteSelectedRow: async ({ commit, getters, dispatch }, payload) => {
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    const row = payload.row
    if (!dataSet) return
    commit('DELETE_SELECTED_ROW', { dataSet, row })
    dispatch('updateUrlQuery', { dataSet: dataSet, selectedRowsDel: [row] })
    dispatch('refreshDataSetDependence', {
      appWindowIdent: payload.appWindowIdent,
      parentDataSetIdent: dataSet.dataSetIdent,
      refreshAggregates: 1,
      sourceKindFrom: 'selectedRows',
    })
  },
  deleteAllSelectedRow: async ({ commit, getters, dispatch }, payload) => {
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    if (!dataSet) return
    if (payload.urlLock !== 1)
      dispatch('updateUrlQuery', {
        dataSet: dataSet,
        selectedRowsDel: dataSet.selectedRows,
      })
    commit('DELETE_ALL_SELECTED_ROW', { dataSet })
  },
  fillDataSetsFromSetters: async ({ dispatch, getters }, payload) => {
    const lookupSetters = getters.getAppWindowLookupSetters(payload.appWindowIdent)
    const { row, sourceKind } = payload
    const lookupAppWindowIdent = getters.getAppWindowIdent(payload.lookupAppWindowIdent)
    if (!payload.lookupIsMultiSelect)
      await lookupSetters.forEach(lookupSetter => {
        dispatch('setDataSetFieldValue', {
          sourceKind: lookupSetter.sourceKindTo,
          appWindowIdent:
            lookupSetter.appWindowIdentTo === lookupAppWindowIdent
              ? payload.lookupAppWindowIdent
              : lookupSetter.appWindowIdentTo,
          dataSet: getters.getDataSet(
            lookupSetter.appWindowIdentTo === lookupAppWindowIdent
              ? payload.lookupAppWindowIdent
              : lookupSetter.appWindowIdentTo,
            lookupSetter.dataSetIdentTo
          ),
          dataFieldIdent: lookupSetter.dataFieldIdentTo,
          dataFieldValue: getters.getDataSetFieldValue(
            lookupSetter.sourceKindFrom,
            getters.getDataSet(
              lookupSetter.appWindowIdentFrom,
              lookupSetter.dataSetIdentFrom
            ),
            lookupSetter.dataFieldIdentFrom
          ),
        })
      })
    if (payload.lookupIsMultiSelect) {
      const whereLists = []
      await lookupSetters.forEach(lookupSetter => {
        const dataSetFrom = getters.getDataSet(
          lookupSetter.appWindowIdentFrom,
          lookupSetter.dataSetIdentFrom
        )
        if (lookupSetter.sourceKindFrom === 'rows' && dataSetFrom.selectedRows.length > 0) {
          dataSetFrom.selectedRows.forEach((row, index) => {
            if (whereLists[index] === undefined) {
              const _keyFieldsValues = getters.getDataSetRowsKeyFieldsValues(
                dataSetFrom,
                row
              )
              whereLists.push({
                [lookupSetter.dataFieldIdentTo]: row[lookupSetter.dataFieldIdentFrom],
                _keyFieldsValues: _keyFieldsValues || undefined,
              })
            } else {
              whereLists[index][[lookupSetter.dataFieldIdentTo]] =
                row[lookupSetter.dataFieldIdentFrom]
            }
          })
        }
      })
      whereLists.forEach(whereListValue => {
        if (sourceKind === 'rows') {
          dispatch('setRowListsFieldValue', {
            dataSet: getters.getDataSet(
              payload.lookupAppWindowIdent,
              payload.lookupDataSetIdent
            ),
            lookupDataFieldIdent: payload.lookupDataFieldIdent,
            whereListValue: whereListValue,
            row:
              row._appWindowIdent === payload.lookupAppWindowIdent &&
                row._dataSetIdent === payload.lookupDataSetIdent
                ? row
                : null,
          })
        } else {
          dispatch('setDataSetWhereListsFieldValue', {
            dataSet: getters.getDataSet(
              payload.lookupAppWindowIdent,
              payload.lookupDataSetIdent
            ),
            lookupDataFieldIdent: payload.lookupDataFieldIdent,
            whereListValue: whereListValue,
          })
        }
      })
    }
  },
  clearDataSetsFromSetters: async ({ dispatch, getters }, payload) => {
    const lookupDef =
      getters.getDataSetLookupDef(
        payload.appWindowIdent,
        payload.dataSetIdent,
        payload.dataFieldIdent
      ) || {}
    const lookupSetters = lookupDef.lookupSetters || []
    await lookupSetters.forEach(lookupSetter => {
      dispatch(
        lookupSetter.sourceKindTo === 'where'
          ? 'setDataSetWhereFieldValue'
          : 'setDataSetRowsFieldValue',
        {
          appWindowIdent: payload.appWindowIdent,
          dataSet: getters.getDataSet(
            lookupSetter.appWindowIdentTo,
            lookupSetter.dataSetIdentTo
          ),
          dataFieldIdent: lookupSetter.dataFieldIdentTo,
          dataFieldValue: null,
        }
      )
    })
  },
  clearLookupDataSetsSelection: async ({ dispatch, getters }, payload) => {
    const lookupDef =
      getters.getDataSetLookupDef(
        payload.appWindowIdent,
        payload.dataSetIdent,
        payload.dataFieldIdent
      ) || {}
    const lookupSettersRefresh = lookupDef.lookupSettersRefresh || []
    await lookupSettersRefresh.forEach(async item => {
      await dispatch('deleteAllSelectedRow', {
        appWindowIdent: item.appWindowIdentFrom,
        dataSetIdent: item.dataSetIdentFrom,
      })
    })
  },
  refreshDataSetFromSetters: async ({ dispatch, getters }, payload) => {
    const lookupSettersRefresh = getters.getAppWindowLookupSettersRefresh(
      payload.appWindowIdent
    )
    for (let i = 0; i < lookupSettersRefresh.length; i++) {
      await dispatch('refreshDataSet', {
        appWindowIdent: lookupSettersRefresh[i].appWindowIdentFrom,
        dataSetIdent: lookupSettersRefresh[i].dataSetIdentFrom,
        refreshAggregates: 1,
      })
    }
  },
  refreshDataSets: async ({ dispatch, getters }, payload) => {
    const appWindowsIdents = []
    if (payload.dataSetIdents !== null) {
      for (let i = 0; i < payload.dataSetIdents.length; i++) {
        if (payload.dataSetIdents[i].appWindowIdent != null) {
          if (
            !appWindowsIdents.find(item => item === payload.dataSetIdents[i].appWindowIdent)
          ) {
            appWindowsIdents.push(payload.dataSetIdents[i].appWindowIdent)
          }
          await dispatch('setDataSetRefReq', {
            appWindowIdent: payload.dataSetIdents[i].appWindowIdent,
            dataSetIdent: payload.dataSetIdents[i].dataSetIdent,
            refreshAggregates: payload.refreshAggregates,
          })
        }
      }
    }
    for (let i = 0; i < appWindowsIdents.length; i++) {
      dispatch('setAppWindowData', {
        appWindowIdent: appWindowsIdents[i],
        query: getters.getRoute ? getters.getRoute.query : null,
      })
    }
  },
  refreshDataSet: ({ dispatch }, payload) => {
    dispatch('refreshDataSets', {
      dataSetIdents: [
        {
          appWindowIdent: payload.appWindowIdent,
          dataSetIdent: payload.dataSetIdent,
        },
      ],
      refreshAggregates: payload.refreshAggregates,
    })
  },
  refreshDataSetDependence: ({ dispatch, getters }, payload) => {
    dispatch('refreshDataSets', {
      dataSetIdents: getters.getDataSetsDependency(
        getters.getDataSet(payload.appWindowIdent, payload.parentDataSetIdent),
        payload.sourceKindFrom
      ),
      refreshAggregates: payload.refreshAggregates,
    })
  },
  setTechnicalFields: ({ commit, getters }, payload) => {
    if (getters.devAppName != payload.devAppName) commit('SET_DEVAPPNAME', payload)
    const dev = payload.DEV == '1'
    if (getters.dev !== dev) commit('SET_ISDEV', { dev })
  },
  addStartWindows: async ({ dispatch, getters, commit }, payload) => {
    if (getters.urlLock) {
      commit('UNLOCK_URL')
      return
    }
    let appWindowIdent = payload.appWindowIdent
    if (appWindowIdent === 'dist') {
      //AKO 20220216 - PO CO TO JEST?
      appWindowIdent = getters.mainAppWindowIdent
    }
    const setActive = payload.setActive
    const path = payload.path
    const query = payload.query
    const serverSide = payload.serverSide
    const { portal } = payload
    if (serverSide) {
      await dispatch('setTechnicalFields', query)
      await dispatch('deleteAppWindowsAll')
    }
    if (appWindowIdent !== getters.mainAppWindowIdent) {
      await dispatch('addAppWindow', {
        appWindowIdent: getters.mainAppWindowIdent,
        setActive: false,
        lookupDef: null,
        addRoutingData: true,
        path: null,
        query: query,
        serverSide: serverSide,
        portal: portal,
        res: payload.res,
        showInManager: false,
      })
    }
    await dispatch('addAppWindow', {
      appWindowIdent: appWindowIdent,
      setActive: setActive,
      lookupDef: null,
      addRoutingData: false,
      path: path,
      query: query,
      serverSide: serverSide,
      portal: portal,
      res: payload.res,
    })
  },
  addLookupWindow: async ({ getters, dispatch, commit }, payload) => {
    const appWindows = Object.values(getters.csAppWindows)
    for (let i = 0; i < appWindows.length; i++) {
      const lookupDef = appWindows[i].lookupDef
      if (
        !lookupDef ||
        lookupDef.closeKind !== 'anotherExpanded' ||
        lookupDef === payload.lookupDef
      )
        continue
      commit('SET_LOOKUP_DEF_VISIBLE', {
        lookupDef: lookupDef,
        lookupVisible: false,
      })
    }
    await dispatch('addAppWindow', payload)
    dispatch('syncDataSetsWithSetters', {
      appWindowIdent: payload.appWindowUniqIdent || payload.appWindowIdent,
    })
  },
  addAppWindow: async ({ commit, dispatch, getters }, payload) => {
    const appWindowIdent = payload.appWindowIdent
    let appWindow = getters.getAppWindow(appWindowIdent, payload.path)
    if (appWindow != null && appWindow.appWindowIdent == getters.mainAppWindowIdent) return

    const lookupDef = payload.lookupDef
    const query = payload.query
    const serverSide = payload.serverSide
    const row = payload.row
    let params = payload.params
    let commands = payload.commands
    const { portal } = payload
    if (appWindow) dispatch('unlockActions', appWindow)
    if (appWindow === undefined) {
      await dispatch('loadAppWindow', payload)
      appWindow = getters.getAppWindow(appWindowIdent, payload.path)
    }
    if (appWindow) {
      const appWindowUniqIdent =
        payload.appWindowUniqIdent ||
        getters.getAppWindowUniqIdentFromAppWindow(appWindow, query)
      if (
        payload.showInManager != undefined &&
        appWindow.showInManager != payload.showInManager
      ) {
        commit('SET_APPWINDOW_SHOW_IN_MANAGER', {
          appWindow,
          showInManager: payload.showInManager,
        })
      }

      if (appWindowUniqIdent !== appWindowIdent) {
        let appWindowNew = getters.getAppWindow(appWindowUniqIdent)
        if (!appWindowNew) {
          appWindowNew = getters.cloneAppWindow(appWindow, appWindowUniqIdent)
          appWindowNew.showInManager =
            payload.showInManager != null ? payload.showInManager : true //AKO 20220302 - maybe we will need to implement showInManagerInitial for source of cloned appWindows
          const mainAppWindowIdent = getters.mainAppWindowIdent

          commit('ADD_APPWINDOW', {
            appWindow: appWindowNew,
            setActive: payload.setActive,
            lookupDef: lookupDef,
            path: payload.path,
            metaInfo: getters.pathMetaInfo(payload.path),
            mainAppWindowIdent: mainAppWindowIdent,
            appWindowIdent: appWindowUniqIdent,
          })
          await dispatch('setActiveMetaInfo', { path: payload.path })

          commit('SET_APPWINDOW_SHOW_IN_MANAGER', { appWindow, showInManager: false })
        }
      }

      if (payload.setActive && appWindowUniqIdent !== getters.activeWindowIdent) {
        await dispatch('setActiveAppWindow', {
          appWindowIdent: appWindowUniqIdent,
          syncRouter: payload.syncRouter != null ? payload.syncRouter : false,
          routerReplace: payload.routerReplace,
        })
      }

      appWindow = getters.getAppWindow(appWindowUniqIdent)
      if (portal) {
        dispatch('addPortal', { portal, appWindowUniqIdent })
      }
      if (!commands) commands = getters.getCommandsFromPath(payload.path)
      if (!params) params = getters.getParamsFromPath(payload.path)
      if (commands) {
        commit('SET_APPWINDOW_COMMANDS', { appWindow, commands })
        await dispatch('processCommands', { commands })
      }

      await dispatch('fillDataSetsFromQuery', {
        appWindowIdent: appWindowUniqIdent,
        query: query,
      })

      if (params && JSON.stringify(params) != JSON.stringify(appWindow.params)) {
        commit('SET_APPWINDOW_PARAMS', { appWindow, params })
        await dispatch('setDataSetRefReqAll', {
          appWindowIdent: appWindowUniqIdent,
          refreshAggregates: 1,
        })
      }
      await dispatch('fillDataSetsFromGetters', {
        appWindowIdent: appWindowUniqIdent,
        dataSetIdent: null,
        row: row,
      })
      await dispatch('setAppWindowData', {
        appWindowIdent: appWindowUniqIdent,
        serverSide: serverSide,
        query: query,
        row: row,
      })
      await dispatch('refreshDataSetFromSetters', {
        appWindowIdent: appWindowUniqIdent,
      })
    }
  },
  syncDataSetsWithSetters: ({ getters, dispatch }, payload) => {
    const { appWindowIdent } = payload
    const lookupDef = getters.getAppWindowLookupDef(appWindowIdent)
    const { row, isMultiSelect, sourceKind } = lookupDef
    if (!row || isMultiSelect !== 1 || sourceKind !== 'rows') return
    const lookupSetters = getters.getAppWindowLookupSetters(appWindowIdent)
    lookupSetters.forEach(setter => {
      if (setter.sourceKindFrom !== 'rows' || setter.sourceKindTo !== 'rows') return
      const rowLists = row._rowLists[setter.dataFieldIdentTo]
      if (!rowLists) return
      dispatch('deleteAllSelectedRow', {
        dataSetIdent: setter.dataSetIdentFrom,
        appWindowIdent: setter.appWindowIdentFrom,
      })
      rowLists.forEach(list => {
        if (!list._keyFieldsValues) return
        let row = new constructors.csRow()
        row[setter.dataFieldIdentFrom] = list[setter.dataFieldIdentTo]
        row = Object.assign(row, list._keyFieldsValues)
        dispatch('addSelectedRow', {
          dataSetIdent: setter.dataSetIdentFrom,
          appWindowIdent: setter.appWindowIdentFrom,
          row: row,
        })
      })
    })
  },
  addPortal: ({ commit }, payload) => {
    commit('ADD_PORTAL', payload)
  },
  delPortal: ({ commit }, payload) => {
    commit('DEL_PORTAL', payload)
  },
  loginUsr: async ({ state, getters, dispatch, rootGetters }, payload) => {
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    const action = getters.getDataSetAction(dataSet, 'csLogin')
    const url = rootGetters['csServerSettings/getAPIDefinitionMethodURL'](
      'login',
      payload.serverSide
    )
    const apiRequestData = {
      connection: getters.getAPIConnectionData,
      route: getters.getAppWindowRoutingData(payload.addRoutingData),
      sessionId: getters.getSessionId(state),
      csAppNameSpacesG: payload.csAppNameSpacesG
        ? payload.csAppNameSpacesG
        : getters.csAppNameSpacesG,
      login: payload.login,
      password: payload.password,
      uPwd: payload.uPwd,
      rPwd: payload.rPwd,
      clientRequest: getters.clientRequest,
    }
    const response = await axios.post(
      url,
      apiRequestData,
      getters.getAPIDataRequestConfig()
    )
    const id = uuid.v1()
    await dispatch('setErrors', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: payload.dataSetIdent,
      errors: response.errors,
      action: action,
      row: payload.row,
      id: id,
      oprIdent: action.actionIdent,
    })
    const data = response.data
    if (data && data.errors) {
      await dispatch('setErrors', {
        appWindowIdent: payload.appWindowIdent,
        dataSetIdent: payload.dataSetIdent,
        errors: data.errors,
        action: action,
        row: payload.row,
        id: id,
        oprIdent: action.actionIdent,
      })
    }
    if (getters.hasError(id)) return
    //
    await dispatch('deleteAppWindowsAll')
    if (await dispatch('checkRouter')) {
      storeUtils.pRouter = storeUtils.pRouter.then(() =>
        storeUtils.router.push(getters.getRoute.from.fullPath)
      )
    }
  },
  logoutUsr: async ({ state, getters, dispatch, rootGetters }, payload) => {
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    const action = getters.getDataSetAction(dataSet, 'csLogout')
    const apiRequestData = {
      sessionId: getters.getSessionId(state),
      csAppNameSpacesG: payload.csAppNameSpacesG,
      connection: getters.getAPIConnectionData,
      route: getters.getAppWindowRoutingData(payload.addRoutingData),
      clientRequest: getters.clientRequest,
    }
    const url = rootGetters['csServerSettings/getAPIDefinitionMethodURL'](
      'logout',
      payload.serverSide
    )
    const response = await axios.post(
      url,
      apiRequestData,
      getters.getAPIDataRequestConfig()
    )
    const data = response.data
    const id = uuid.v1()
    await dispatch('setErrors', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: payload.dataSetIdent,
      errors: response.errors,
      action: action,
      row: payload.row,
      id: id,
      oprIdent: action.actionIdent,
    })
    if (data && data.errors) {
      await dispatch('setErrors', {
        appWindowIdent: payload.appWindowIdent,
        dataSetIdent: payload.dataSetIdent,
        errors: data.errors,
        action: action,
        row: payload.row,
        id: id,
        oprIdent: action.actionIdent,
      })
    }
    if (getters.hasError(id)) return
    await dispatch('deleteAppWindowsAll')
    if (await dispatch('checkRouter'))
      storeUtils.pRouter = storeUtils.pRouter.then(() => storeUtils.router.go())
  },
  gotoLocation: ({ dispatch }, payload) => {
    const location = payload.location
    const res = payload.res
    const r = new RegExp('^(?:[a-z]+:)?//', 'i')
    if (res != null && location != null && payload.redirect === 1) {
      res.redirect(location)
    } else if (window != null && location != null && payload.redirect === 1) {
      window.location.replace(location)
    } else if (location != null && r.test(location.path)) {
      window.location = location.path
    } else if (dispatch('checkRouter')) {
      storeUtils.pRouter = storeUtils.pRouter.then(() =>
        storeUtils.router.push(payload.location)
      )
    }
  },
  playAudio: ({ getters, commit }, payload) => {
    const { url } = payload
    if (!url) return
    let audio = getters.getAudio(url)
    if (!audio) {
      audio = new Audio(url)
      payload.audio = audio
      commit('ADD_AUDIO', payload)
    }
    if (payload.replay == 1) audio.load()
    audio.play()
  },
  pauseAudio: ({ getters }, payload) => {
    const { url } = payload
    if (!url) return
    const audio = getters.getAudio(url)
    if (!audio) return
    audio.pause()
  },
  unlockActions: ({ commit }, appWindow) => {
    if (appWindow.dataSets)
      for (const dataSet in appWindow.dataSets) {
        if (dataSet.actions)
          dataSet.actions.forEach(action => {
            commit('SET_DATASET_ACTION_STATE', { action: action, state: 1 })
          })
      }
  },
  loadAppWindow: async ({ commit, state, getters, dispatch, rootGetters }, payload) => {
    const refreshParamsJSON = getters.getRefreshParams()
    const getMetaInfo = state.paths[payload.path] ? false : true
    const request = {
      serverSide: payload.serverSide,
      csAppNameSpacesG: getters.csAppNameSpacesG,
      appWindowIdent: payload.appWindowIdent,
      sessionId: getters.getSessionId(state),
      viewHTMLFromFile: getters.viewHTMLFromFile,
      connection: refreshParamsJSON ? getters.getAPIConnectionData : undefined,
      route: getters.getAppWindowRoutingData(payload.addRoutingData || payload.serverSide),
      path: payload.path,
      refreshParamsJSON: refreshParamsJSON ? refreshParamsJSON : undefined,
      loginProvider: payload.serverSide
        ? rootGetters['csServerSettings/getLoginProvider']
        : null,
      getMetaInfo: getMetaInfo,
      clientRequest: getters.clientRequest,
    }
    const res = payload.res
    const url = rootGetters['csServerSettings/getAPIDefinitionMethodURL'](
      'getDefinition',
      payload.serverSide
    )
    const response = await axios.post(url, [request], getters.getAPIDataRequestConfig())
    await dispatch('setErrors', {
      errors: response.errors,
    })
    const data = response.data
    await dispatch('processCommands', { commands: data.commands, res: res })
    if (data.errors)
      await dispatch('setErrors', {
        errors: data.errors,
      })
    commit('SET_SESSIONID', { sessionId: response.data.sessionId })
    if (response.data.definition === undefined || response.data.definition === null) {
      commit('ADD_LOG', {
        messageText:
          'Error loading window: ' +
          payload.appWindowIdent +
          ' serverSide: ' +
          payload.serverSide +
          ' url: ' +
          url +
          ' request: ' +
          JSON.stringify(request) +
          ':' +
          JSON.stringify(response.data),
      })
      return
    }
    const appWindow = getters.getAppWindowFromDefinition(
      payload.appWindowIdent,
      response.data.definition,
      payload.lookupDef,
      payload.showInManager
    )
    if (appWindow == null || appWindow.appWindowIdent == null) return
    //
    const setActive = payload.setActive
    const lookupDef = payload.lookupDef
    const path = response.data.pathChange != null ? response.data.pathChange : payload.path
    payload.path = path
    const metaInfo = response.data.metaInfo
    const mainAppWindowIdent = getters.mainAppWindowIdent
    const appWindowIdent = appWindow.appWindowIdent
    commit('ADD_APPWINDOW', {
      appWindow,
      setActive,
      lookupDef,
      path,
      metaInfo,
      mainAppWindowIdent,
      appWindowIdent,
    })
    await dispatch('setActiveMetaInfo', { path })
    if (!await dispatch('checkRouter')) return
    if (
      response.data.pathChange != null &&
      storeUtils.router != null &&
      storeUtils.router.currentRoute.path !== path
    ) {
      storeUtils.pRouter = storeUtils.pRouter.then(() => {
        storeUtils.router.replace({ path: path })
      })
    }
  },
  deleteAppWindow: async ({ commit, dispatch, getters, state }, payload) => {
    if (payload === null) throw Error('[deleteAppWindow] -  payload is null.')
    if (payload.appWindowIdent === null)
      throw Error('[deleteAppWindow] -  appWindowIdent is null.')
    //
    let appWindowDelOrdInManager = null
    if (getters.activeWindowIdent == payload.appWindowIdent) {
      const appWindowDel = state.csAppWindows[payload.appWindowIdent]
      appWindowDelOrdInManager =
        appWindowDel.showInManager == true ? appWindowDel.ordInManager : null
    }
    //
    commit('DELETE_APPWINDOW', payload)
    //
    if (appWindowDelOrdInManager != null) {
      let ordInManagerMax = -1
      let appWindowIdentOrdInManagerMax = null
      //
      let ordInManagerSetActive = -1
      let appWindowIdentSetActive = null
      //
      for (const appWindowsKey in state.csAppWindows) {
        const appWindow = state.csAppWindows[appWindowsKey]

        if (
          appWindow.showInManager == true &&
          appWindow.ordInManager != null &&
          appWindow.ordInManager > ordInManagerMax
        ) {
          ordInManagerMax = appWindow.ordInManager
          appWindowIdentOrdInManagerMax = appWindowsKey
        }
        if (
          appWindow.showInManager == true &&
          appWindow.ordInManager != null &&
          appWindow.ordInManager > appWindowDelOrdInManager
        ) {
          if (
            appWindowIdentSetActive == null ||
            appWindow.ordInManager < ordInManagerSetActive
          ) {
            appWindowIdentSetActive = appWindowsKey
            ordInManagerSetActive = appWindow.ordInManager
          }
        }
      }
      if (appWindowIdentSetActive == null) {
        appWindowIdentSetActive = appWindowIdentOrdInManagerMax
      }
      await dispatch('setActiveAppWindow', {
        appWindowIdent: appWindowIdentSetActive,
        syncRouter: true,
        routerReplace: payload.routerReplace,
      })
    }
  },
  reloadAppWindow: async ({ commit, dispatch, getters }, payload) => {
    if (payload === null) throw Error('[reloadAppWindow] -  payload is null.')
    if (payload.appWindowIdent === null)
      throw Error('[reloadAppWindow] -  appWindowIdent is null.')
    //
    await dispatch('deleteAppWindow', {
      appWindowIdent: payload.appWindowUniqIdent || payload.appWindowIdent,
      routerReplace: true,
    })
    //
    if (
      payload.appWindowUniqIdent != null &&
      payload.appWindowIdent != null &&
      payload.appWindowUniqIdent != payload.appWindowIdent
    ) {
      await dispatch('deleteAppWindow', {
        appWindowIdent: payload.appWindowIdent,
        routerReplace: true,
      })
    }
    //
    try {
      commit('LOCK_URL')
      //
      await dispatch('addAppWindow', {
        appWindowIdent: payload.appWindowIdent,
        setActive: true,
        lookupDef: null,
        addRoutingData: true,
        path: null,
        query: payload.query || getters.getRoute.query,
        syncRouter: true,
        routerReplace: true,
      })
      //
      await dispatch('syncRouterWithActiveAppWindow', {
        appWindowIdent: payload.appWindowUniqIdent || payload.appWindowIdent,
        routerReplace: true,
      })
    } finally {
      commit('UNLOCK_URL')
    }
  },
  setActiveAppWindow: async ({ commit, dispatch }, payload) => {
    if (payload === null) throw Error('[setActiveAppWindow] -  payload is null.')
    //
    commit('SETACTIVE_APPWINDOW', payload)
    //
    if (payload.syncRouter == true) {
      await dispatch('syncRouterWithActiveAppWindow', {
        appWindowIdent: payload.appWindowIdent,
        routerReplace: payload.routerReplace,
      })
    }
  },
  syncRouterWithActiveAppWindow: async ({ dispatch, getters, state }, payload) => {
    if (state != null && state.paths != null) {
      let newPath
      if (payload.appWindowIdent == null) {
        newPath = '/'
        if (await dispatch('checkRouter')) {
          storeUtils.pRouter = storeUtils.pRouter.then(() => {
            if (storeUtils.router.currentRoute.path != newPath) {
              storeUtils.router.push({
                path: newPath,
              })
            }
          })
        }
      } else {
        for (const path of Object.keys(state.paths)) {
          if (state.paths[path].appWindowIdent == payload.appWindowIdent) {
            newPath = path
            break
          }
        }
        //
        if (newPath != null && await dispatch('checkRouter')) {
          storeUtils.pRouter = storeUtils.pRouter.then(() => {
            const appWindow = getters.getAppWindow(payload.appWindowIdent)
            if (
              storeUtils.router.currentRoute.path != newPath ||
              storeUtils.router.currentRoute.path == newPath &&
              JSON.stringify(storeUtils.router.currentRoute.query) !=
              JSON.stringify(appWindow.query)
            ) {
              if (payload.routerReplace == true) {
                storeUtils.router.replace({
                  path: newPath,
                  query: appWindow.query,
                })
              } else {
                storeUtils.router.push({
                  path: newPath,
                  query: appWindow.query,
                })
              }
            }
          })
        }
      }
      //
    }
  },
  dataSetPageChange: ({ dispatch }, payload) => {
    dispatch('refreshDataSet', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: payload.dataSetIdent,
      refreshAggregates: 0,
    })
  },
  setDataSetPageNo: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const pagesCount = getters.getDataSetPagesCount(dataSet)
    const pageNo = payload.pageNo
    if (pagesCount > 0 && pageNo > pagesCount) return
    if (pageNo < 1) return
    commit('SET_DATASET_PAGENOREF', { dataSet, pagesCount, pageNo })
    dispatch('dataSetPageChange', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: dataSet.dataSetIdent,
    })
  },
  setDataSetPivotColsPageNo: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const pagesCount = getters.getDataSetPivotColsPagesCount(dataSet)
    const pageNo = payload.pageNo
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    commit('SET_DATASET_PIVOT_COLS_PAGENO', {
      dataSet,
      layout,
      pagesCount,
      pageNo,
    })
    dispatch('dataSetPageChange', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: dataSet.dataSetIdent,
    })
  },
  setDataSetPivotRowsPageNo: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const pagesCount = getters.getDataSetPivotRowsPagesCount(dataSet)
    const pageNo = payload.pageNo
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    commit('SET_DATASET_PIVOT_ROWS_PAGENO', {
      dataSet,
      layout,
      pagesCount,
      pageNo,
    })
    dispatch('dataSetPageChange', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: dataSet.dataSetIdent,
    })
  },
  setDataSetPageNoFirst: ({ dispatch }, payload) => {
    payload.pageNo = 1
    dispatch('setDataSetPageNo', payload)
  },
  setDataSetPivotColsPageNoFirst: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    commit('SET_DATASET_PIVOT_COLS_PAGENOFIRST', { dataSet, layout })
    dispatch('dataSetPageChange', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: dataSet.dataSetIdent,
    })
  },
  setDataSetPivotRowsPageNoFirst: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    commit('SET_DATASET_PIVOT_ROWS_PAGENOFIRST', { dataSet, layout })
    dispatch('dataSetPageChange', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: dataSet.dataSetIdent,
    })
  },
  setDataSetPageNoLast: ({ getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    payload.pageNo = getters.getDataSetPagesCount(dataSet)
    dispatch('setDataSetPageNo', payload)
  },
  setDataSetPivotColsPageNoLast: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const pagesCount = getters.getDataSetPivotColsPagesCount(dataSet)
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    commit('SET_DATASET_PIVOT_COLS_PAGENOLAST', {
      dataSet,
      layout,
      pagesCount,
    })
    dispatch('dataSetPageChange', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: dataSet.dataSetIdent,
    })
  },
  setDataSetPivotRowsPageNoLast: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const pagesCount = getters.getDataSetPivotRowsPagesCount(dataSet)
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    commit('SET_DATASET_PIVOT_ROWS_PAGENOLAST', {
      dataSet,
      layout,
      pagesCount,
    })
    dispatch('dataSetPageChange', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: dataSet.dataSetIdent,
    })
  },
  setDataSetPageNoNext: ({ getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const pagesCount = getters.getDataSetPagesCount(dataSet)
    if (dataSet.pageNo >= pagesCount) return
    payload.pageNo = dataSet.pageNo + 1
    dispatch('setDataSetPageNo', payload)
  },
  setDataSetPivotColsPageNoNext: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const pagesCount = getters.getDataSetPivotColsPagesCount(dataSet)
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    commit('SET_DATASET_PIVOT_COLS_PAGENONEXT', {
      dataSet,
      layout,
      pagesCount,
    })
    commit('SET_DATASET_SCROLL_DIRECTION', {
      dataSet: dataSet,
      scrollDirection: dataSetUtils.scrollDirection.right,
    })
    dispatch('dataSetPageChange', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: dataSet.dataSetIdent,
    })
  },
  setDataSetPivotRowsPageNoNext: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const pagesCount = getters.getDataSetPivotRowsPagesCount(dataSet)
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    commit('SET_DATASET_PIVOT_ROWS_PAGENONEXT', {
      dataSet,
      layout,
      pagesCount,
    })
    commit('SET_DATASET_SCROLL_DIRECTION', {
      dataSet: dataSet,
      scrollDirection: dataSetUtils.scrollDirection.bottom,
    })
    dispatch('dataSetPageChange', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: dataSet.dataSetIdent,
    })
  },
  setDataSetPageNoPrev: ({ dispatch }, payload) => {
    const dataSet = payload.dataSet
    if (dataSet.pageNo <= 1) return
    payload.pageNo = dataSet.pageNo - 1
    dispatch('setDataSetPageNo', payload)
  },
  setDataSetPivotColsPageNoPrev: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    commit('SET_DATASET_PIVOT_COLS_PAGENOPREV', { dataSet, layout })
    commit('SET_DATASET_SCROLL_DIRECTION', {
      dataSet: dataSet,
      scrollDirection: dataSetUtils.scrollDirection.left,
    })
    dispatch('dataSetPageChange', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: dataSet.dataSetIdent,
    })
  },
  setDataSetPivotRowsPageNoPrev: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    commit('SET_DATASET_PIVOT_ROWS_PAGENOPREV', { dataSet, layout })
    commit('SET_DATASET_SCROLL_DIRECTION', {
      dataSet: dataSet,
      scrollDirection: dataSetUtils.scrollDirection.top,
    })
    dispatch('dataSetPageChange', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: dataSet.dataSetIdent,
    })
  },
  setDataSetLayoutIdent: ({ commit }, payload) => {
    commit('SET_LAYOUT_IDENT', payload)
  },
  setDataSetSortIdent: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const sortIdent = payload.sortIdent
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    commit('SET_LAYOUT_SORTIDENT', { layout, sortIdent })
    dispatch('setDataSetWhereFromAttrib', {
      dataSet: dataSet,
      attribName: 'sortIdent',
      attribValue: sortIdent,
    })
    dispatch('setDataSetPageNoFirst', {
      appWindowIdent: payload.appWindowIdent,
      dataSet: payload.dataSet,
    })
  },
  setDataSetPageSizesIdent: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const pageSizesIdent = payload.pageSizesIdent
    const pageSizes = getters.getDataSetPageSizes(dataSet, pageSizesIdent)
    let pageSize
    if (pageSizes !== null && pageSizes !== undefined) {
      pageSize = pageSizes.pageSize
    }
    commit('SET_DATASET_PAGESIZESIDENT', {
      dataSet,
      pageSizesIdent,
      pageSize,
    })
    dispatch('setDataSetWhereFromAttrib', {
      dataSet: dataSet,
      attribName: 'pageSizeIdent',
      attribValue: pageSizesIdent,
    })
    dispatch('setDataSetPageNoFirst', {
      appWindowIdent: payload.appWindowIdent,
      dataSet: payload.dataSet,
    })
  },
  clearDataSet: ({ commit, getters }, payload) => {
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    commit('CLEAR_DATASET', { dataSet, layout })
  },
  incDataSetRefCount: ({ commit, getters }, payload) => {
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    commit('INC_DATASET_REFCOUNT', { dataSet })
  },
  deleteDataSetWhereListsItem: async ({ commit, getters, dispatch }, payload) => {
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    const { dataFieldIdent, row, sourceKind } = payload
    if (sourceKind === 'rows') {
      if (row != null) commit('DELETE_ROW_LISTS', { row, dataFieldIdent })
    }
    else commit('DELETE_DATASET_WHERE_LISTS', { dataSet, dataFieldIdent })
    dispatch('refreshDataSetDependence', {
      appWindowIdent: payload.appWindowIdent,
      parentDataSetIdent: dataSet.dataSetIdent,
      refreshAggregates: 1,
      sourceKindFrom: 'selectedRows',
    })
  },
  deleteDataSetWhereListItem: async ({ commit, getters, dispatch }, payload) => {
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    const { dataFieldIdent, dataFieldValue, row, sourceKind } = payload
    if (sourceKind === 'rows')
      commit('DELETE_ROW_LIST_ITEM', {
        row,
        dataFieldIdent,
        dataFieldValue,
      })
    else
      commit('DELETE_DATASET_WHERE_LIST_ITEM', {
        dataSet,
        dataFieldIdent,
        dataFieldValue,
      })
    const lookupDef = getters.getDataSetLookupDef(
      payload.appWindowIdent,
      payload.dataSetIdent,
      dataFieldIdent
    )
    if (lookupDef && lookupDef.isMultiSelect && payload._keyFieldsValues) {
      const lookupSettersRefresh = lookupDef.lookupSettersRefresh || []
      await lookupSettersRefresh.forEach(item => {
        dispatch('deleteSelectedRow', {
          appWindowIdent: item.appWindowIdentFrom,
          dataSetIdent: item.dataSetIdentFrom,
          row: payload._keyFieldsValues,
        })
      })
    }
  },
  setDataSetBusy: ({ commit, getters }, payload) => {
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    commit('SET_DATASET_BUSY', {
      appWindowIdent: payload.appWindowIdent,
      dataSet: dataSet,
    })
  },
  printDevInfo: ({ getters }, payload) => {
    if (!payload || !getters.isDev) return
    const { dataSetIdent, appWindowIdent } = payload
    const refConditions = getters.getDataSetRefConditions(
      getters.getDataSet(appWindowIdent, dataSetIdent)
    )
    if (refConditions.length < 1) return
    let result = 'Refresh start\n'
    result += '  appWindowIdent: ' + appWindowIdent + '\n'
    result += '  dataSetIdent: ' + dataSetIdent + '\n'
    result += '  conditions:\n'

    refConditions.forEach(msg => {
      result += '    ' + JSON.stringify(msg) + '\n'
    })
    result += 'Refresh end\n'
    /* eslint-disable no-console */
    console.log(result)
    /* eslint-enable no-console */
  },
  setAppWindowData: async ({ dispatch, commit, getters, rootGetters }, payload) => {
    let ord = 0
    const appWindowIdent = payload.appWindowIdent
    if (appWindowIdent == null) return
    const query = payload.query
    const maxOrd = getters.getAppWindowDataSetsMaxOrd(appWindowIdent)
    let dataSetList = null
    let dataSetIdent = null
    let dataToPost = null
    let response = null
    let data = null
    const row = payload.row
    const serverSide = payload.serverSide

    while (ord < maxOrd) {
      ord++
      dataSetList = getters.getDataSetsToRefresh(appWindowIdent, ord)
      if (dataSetList == null || dataSetList.length == 0) {
        continue
      }
      for (let i = 0; i < dataSetList.length; i++) {
        dataSetIdent = null
        dataSetIdent = dataSetList[i].dataSetIdent
        dispatch('printDevInfo', {
          appWindowIdent: appWindowIdent,
          dataSetIdent: dataSetIdent,
        })
        await dispatch('setDataSetBusy', {
          appWindowIdent: appWindowIdent,
          dataSetIdent: dataSetIdent,
        })
        await dispatch('fillDataSetWhere', {
          appWindowIdent: appWindowIdent,
          dataSetIdent: dataSetIdent,
        })
      }
      for (let i = 0; i < dataSetList.length; i++) {
        await dispatch('incDataSetRefCount', {
          appWindowIdent: appWindowIdent,
          dataSetIdent: dataSetIdent,
        })
      }
      response = null
      dataToPost = null
      dataToPost = await getters.getAPIDataRequest(appWindowIdent, dataSetList, serverSide)
      response = await axios.post(
        rootGetters['csServerSettings/getAPIDataMethodURL']('getData', serverSide),
        dataToPost,
        getters.getAPIDataRequestConfig()
      )
      await dispatch('setErrors', {
        appWindowIdent: appWindowIdent,
        dataSetIdent: dataSetIdent,
        errors: response.errors,
      })
      //
      data = null
      data = response.data

      //To handle logout when session expires
      await dispatch('processCommands', { commands: data.commands })
      //
      if (data !== null) {
        const wasErrors = await dispatch('setErrors', {
          appWindowIdent: appWindowIdent,
          dataSetIdent: dataSetIdent,
          errors: data.errors,
        })
        if (!wasErrors) {
          commit('SET_APPWINDOWDATA', {
            appWindowIdent,
            dataSetList,
            data,
            query,
          })
        }
      }
      for (let i = 0; i < dataSetList.length; i++) {
        dispatch('updateDataSetPageNo', {
          dataSet: getters.getDataSet(appWindowIdent, dataSetList[i].dataSetIdent),
        })
        if (dataSetList[i].crLocData === 1) {
          await dispatch('fillDataSetsFromGetters', {
            appWindowIdent: appWindowIdent,
            dataSetIdent: dataSetList[i].dataSetIdent,
            row: row,
          })
        }
        if (!getters.getDataSetPivotIsEmpty(dataSetList[i])) {
          let pivotCurrentRowIndex = 0
          let pivotCurrentColIndex = 0
          let pivotCurrentDataFieldIdent = getters.getDataSetAggrFields(dataSetList[i])[0]
            .dataFieldIdent
          switch (dataSetList[i].scrollDirection) {
            case dataSetUtils.scrollDirection.right:
              pivotCurrentRowIndex =
                dataSetList[i].pivotPrevRowIndex < dataSetList[i].pivotRows.length
                  ? dataSetList[i].pivotPrevRowIndex
                  : 0
              pivotCurrentDataFieldIdent = dataSetList[i].pivotPrevDataFieldIdent
              break
            case dataSetUtils.scrollDirection.left:
              pivotCurrentRowIndex =
                dataSetList[i].pivotPrevRowIndex < dataSetList[i].pivotRows.length
                  ? dataSetList[i].pivotPrevRowIndex
                  : 0
              pivotCurrentDataFieldIdent = dataSetList[i].pivotPrevDataFieldIdent
              break
            case dataSetUtils.scrollDirection.bottom:
              pivotCurrentColIndex =
                dataSetList[i].pivotPrevColIndex < dataSetList[i].pivotCols.length
                  ? dataSetList[i].pivotPrevColIndex
                  : 0
              break
            case dataSetUtils.scrollDirection.top:
              pivotCurrentColIndex =
                dataSetList[i].pivotPrevColIndex < dataSetList[i].pivotCols.length
                  ? dataSetList[i].pivotPrevColIndex
                  : 0
              break
          }
          await dispatch('setDataSetCurrentPivotIndex', {
            appWindowIdent: appWindowIdent,
            dataSetIdent: dataSetList[i].dataSetIdent,
            pivotCurrentRowIndex: pivotCurrentRowIndex,
            pivotCurrentColIndex: pivotCurrentColIndex,
            pivotCurrentDataFieldIdent: pivotCurrentDataFieldIdent,
          })
        }
        ////
        const dataSetsTo = dataSetList[i].dataSetsTo
        if (dataSetsTo != null) {
          dataSetsTo.forEach(async function (dataSetsToItem) {
            if (dataSetsToItem.sourceKindTo != 'where') return
            const dataSetTo = getters.getDataSet(
              dataSetsToItem.appWindowIdentTo,
              dataSetsToItem.dataSetIdentTo
            )
            if (dataSetTo == null || dataSetTo.refReq === 1 || dataSetTo.ord <= ord) return
            const whereFieldsIdent = getters.getDataSetWhereFieldsIdentsDataField(
              dataSetTo,
              dataSetsToItem.dataFieldIdentTo
            )
            if (
              whereFieldsIdent != null &&
              whereFieldsIdent.refreshDependencyKind !== 'none'
            ) {
              await dispatch('setDataSetRefReq', {
                appWindowIdent: dataSetsToItem.appWindowIdentTo,
                dataSetIdent: dataSetsToItem.dataSetIdentTo,
                refreshAggregates: 1,
                refCondition: {
                  dataSetIdentFrom: dataSetList[i].dataSetIdent,
                  dataSetsTo: dataSetsToItem,
                },
              })
            }
          })
        }
        ////
      }
    }
  },
  updateDataSetPageNo: ({ dispatch, commit }, payload) => {
    const { dataSet } = payload
    if (dataSet == null) return
    commit('SET_DATASET_PAGENO', {
      dataSet: dataSet,
      pageNo: dataSet.pageNoRef,
    })
    commit('SET_DATASET_PAGENOREF', { dataSet: dataSet, pageNo: 1 })
    dispatch('setDataSetWhereFromAttrib', {
      dataSet: dataSet,
      attribName: 'pageNo',
      attribValue: dataSet.pageNo,
    })
    dispatch('updateUrlQuery', { dataSet })
  },
  invokeActionSetOprState: ({ commit }, payload) => {
    if (payload.oprState == null) return
    //
    if (payload.invokeActionLockKind === 1) {
      commit('SET_DATASET_ROW_OPR_STATE', {
        row: payload.row,
        oprState: payload.oprState,
      })
    } else if (payload.invokeActionLockKind === 2) {
      commit('SET_DATASET_OPR_STATE', {
        dataSet: payload.dataSet,
        oprState: payload.oprState,
      })
    } else if (payload.invokeActionLockKind === 3) {
      if (payload.appWindow != null) {
        for (const dataSetIdent in payload.appWindow.dataSets) {
          const dataSet = payload.appWindow.dataSets[dataSetIdent]
          commit('SET_DATASET_OPR_STATE', {
            dataSet: dataSet,
            oprState: payload.oprState,
          })
        }
      }
    }
  },
  invokeAction: async ({ dispatch, commit, getters, state, rootGetters }, payload) => {
    if (payload.actionIdent == null) return
    //
    const appWindowIdent = getters.getAppWindowIdent(payload.appWindowIdent)
    const appWindow = getters.getAppWindow(appWindowIdent)
    const csAppNameSpacesG = appWindow.csAppNameSpacesG

    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    let rowIndex = payload.rowIndex
    let row = payload.row
    const action = getters.getDataSetAction(dataSet, payload.actionIdent)
    if (action == null) return
    //
    let actionState = getters.getDataSetActionState(dataSet, payload.actionIdent, row)
    //
    if (actionState === 0) return
    //
    if (
      dataSet?._technical.oprState == dataSetUtils.oprState.inProgress ||
      row?._technical.oprState == dataSetUtils.oprState.inProgress
    ) {
      return
    }
    //
    let where, commands
    try {
      await dispatch('invokeActionSetOprState', {
        invokeActionLockKind: action.invokeActionLockKind,
        appWindow: appWindow,
        dataSet: dataSet,
        row: row,
        oprState: dataSetUtils.oprState.inProgress,
      })
      //
      if (action.addNewRow && payload.execute !== 1) {
        await dispatch('addRow', {
          appWindowIdent: payload.appWindowIdent,
          dataSet: dataSet,
          fieldsIdents: action.fieldsIdents,
          row: payload.row,
        })
        rowIndex = dataSet.rows.length - 1
        row = dataSet.rows[rowIndex]
      }
      if (action.showView === 1 && payload.execute !== 1) {
        const activeAppWindow = getters.getAppWindow(getters.activeWindowIdent)
        await dispatch('createRowBackup', { row })
        commit('ADD_MODAL_WINDOW', {
          appWindow: activeAppWindow,
          data: {
            appWindowIdent: payload.appWindowIdent,
            dataSetIdent: payload.dataSetIdent,
            actionIdent: payload.actionIdent,
            onCFDataFieldIdent: payload.onCFDataFieldIdent,
            row: row,
            rowIndex: rowIndex,
            addNewRow: action.addNewRow,
          },
        })
        return
      }
      commit('SET_DATASET_ACTION_STATE', { action: action, row: row, state: 0 })
      commit('CLEAR_ERRORFIELDS', {
        dataSet: dataSet,
      })
      //
      if (action.uploadFiles === 1) {
        await dispatch('putAllFiles', { dataSet })
        if (!getters.isFilesUploaded(dataSet)) {
          commit('SET_DATASET_ACTION_STATE', {
            action: action,
            row: row,
            state: -1,
          })
          return
        }
      }
      //
      if (payload.actionIdent === 'csLogin') {
        const loginPayload = {
          csAppNameSpacesG: csAppNameSpacesG,
          appWindowIdent: appWindowIdent,
          dataSetIdent: payload.dataSetIdent,
          login: row ? row.login : null,
          password: row ? row.password : null,
          uPwd: row ? row.uPwd : null,
          router: payload.router,
          row: payload.row,
        }
        await dispatch('loginUsr', loginPayload)
      } else if (payload.actionIdent === 'csLogout') {
        const logoutPayload = {
          csAppNameSpacesG: csAppNameSpacesG,
          appWindowIdent: appWindowIdent,
          dataSetIdent: payload.dataSetIdent,
          row: payload.row,
          addRoutingData: payload.addRoutingData,
          serverSide: payload.serverSide,
        }
        await dispatch('logoutUsr', logoutPayload)
      } else if (
        action.kind === 'exportXLSX' ||
        action.kind === 'exportCSV' ||
        action.kind === 'exportTXT'
      ) {
        await dispatch('exportData', payload)
      } else {
        where = dataSet ? dataSet.where : null
        const whereLists = dataSet ? dataSet.whereLists : null
        const sources = getters.getDataSetActionSourcesValues(action)
        const refreshParamsJSON = getters.getRefreshParams()
        let layout
        if (action.addLayout === 1) {
          const layoutIdent = payload.layoutIdent
          if (dataSet && layoutIdent)
            layout = getters.getDataSetLayout(dataSet, layoutIdent)
        }
        //
        const apiRequestData = {
          csAppNameSpacesG: csAppNameSpacesG,
          appWindowIdent: appWindowIdent,
          dataSetIdent: payload.dataSetIdent,
          actionIdent: payload.actionIdent,
          sessionId: getters.getSessionId(state),
          connection: refreshParamsJSON ? getters.getAPIConnectionData : undefined,
          where: getters.undefinedWhenObjNull(
            getters.getDataSetActionWhere(action, where),
            true
          ),
          whereLists: getters.undefinedWhenObjNull(whereLists, true),
          selectedRows: getters.undefinedWhenArrayEmpty(
            getters.getDataSetActionSelectedRows(action, dataSet)
          ),
          row: getters.undefinedWhenObjNull(
            getters.getDataSetActionRow(action, payload.row),
            true
          ),
          sources: getters.undefinedWhenObjNull(sources, true),
          onCFDataFieldIdent: payload.onCFDataFieldIdent
            ? payload.onCFDataFieldIdent
            : undefined,
          refreshParamsJSON: refreshParamsJSON ? refreshParamsJSON : undefined,
          files: dataSet.files,
          layout: layout ? layout : undefined,
          path: storeUtils.router.currentRoute.path,
          clientRequest: getters.clientRequest,
        }
        const response = await axios.post(
          rootGetters['csServerSettings/getAPIDataMethodURL'](
            'invokeAction',
            payload.serverSide
          ),
          apiRequestData,
          getters.getAPIDataRequestConfig()
        )
        //
        await dispatch('clearHiddenErrors', { state: state, dispatch: dispatch })
        await dispatch('setErrors', {
          appWindowIdent: payload.appWindowIdent,
          dataSetIdent: payload.dataSetIdent,
          row: payload.row,
          errors: response.errors,
          action: action,
          oprIdent: payload.actionIdent,
        })
        const data = response.data
        if (data !== null) {
          await dispatch('clearHiddenErrors', {
            state: state,
            dispatch: dispatch,
          })
          await dispatch('setErrors', {
            appWindowIdent: payload.appWindowIdent,
            dataSetIdent: payload.dataSetIdent,
            row: payload.row,
            errors: data.errors,
            action: action,
            oprIdent: payload.actionIdent,
          })
          commands = data.commands
        }
      }
      actionState = getters.getDataSetActionState(dataSet, payload.actionIdent, row)
      if (actionState !== -1) {
        commit('SET_DATASET_ACTION_STATE', {
          action: action,
          row: payload.row,
          state: 1,
        })
        if (action.removeFiles === 1) commit('DELETE_ALL_FILES', { dataSet })
        if (payload.execute === 1 && payload.modalWindow)
          await dispatch('deleteModalWindow', payload.modalWindow)
        if (action.clearSelectedRows === 1) commit('DELETE_ALL_SELECTED_ROW', { dataSet })
      }
    } finally {
      await dispatch('invokeActionSetOprState', {
        invokeActionLockKind: action.invokeActionLockKind,
        appWindow: appWindow,
        dataSet: dataSet,
        row: row,
        oprState: dataSetUtils.oprState.finished,
      })
    }
    dispatch('processCommands', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: payload.dataSetIdent,
      commands: commands,
      row: payload.row,
      where: where,
      router: payload.router,
      window: payload.window,
    })
  },
  exportData: async ({ getters, dispatch, rootGetters }, payload) => {
    if (!payload.appWindowIdent || !payload.dataSetIdent) return
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    const action = getters.getDataSetAction(dataSet, payload.actionIdent)
    const request = getters.getAPIExportDataRequest(
      payload.appWindowIdent,
      payload.dataSetIdent,
      action.kind
    )
    const response = await axios.post(
      rootGetters['csServerSettings/getAPIDataMethodURL']('exportData', payload.serverSide),
      request,
      getters.getAPIDataRequestConfig()
    )
    await dispatch('setErrors', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: payload.dataSetIdent,
      errors: response.errors,
      action: action,
      row: payload.row,
      oprIdent: action.actionIdent,
    })
    const data = response.data
    if (!data) return
    if (data.errors) {
      await dispatch('setErrors', {
        appWindowIdent: payload.appWindowIdent,
        dataSetIdent: payload.dataSetIdent,
        errors: data.errors,
        action: action,
        row: payload.row,
        oprIdent: action.actionIdent,
      })
    }
    if (action.state !== -1 && payload.window && data.fileUrl)
      payload.window.location.href =
        rootGetters['csServerSettings/getAPIDataMethodURL'](
          'content/download',
          payload.serverSide
        ) + data.fileUrl
  },
  createRowBackup: ({ commit }, payload) => {
    if (!payload) return
    const { row } = payload
    commit('CLEAR_ROW_BACKUP', payload)
    const rowBackup = JSON.parse(JSON.stringify(row))
    commit('ADD_ROW_BACKUP', { row, rowBackup })
  },
  restoreRowBackup: ({ commit, getters }, payload) => {
    if (!payload) return
    const { dataSet, row } = payload
    const rowIndex = getters.getDataSetRowIndex(dataSet, row)
    commit('RESTORE_ROW_BACKUP', { dataSet, rowIndex })
  },
  downloadFile: async ({ rootGetters }, payload) => {
    if (!payload.window || payload.remoteFileName == null) return
    const localFileName =
      payload.localFileName == null ? payload.remoteFileName : payload.localFileName
    payload.window.location.href =
      rootGetters['csServerSettings/getAPIDataMethodURL'](
        'content/download',
        payload.serverSide
      ) +
      '/' +
      payload.remoteFileName +
      '/' +
      localFileName
  },
  addError: async ({ dispatch }, payload) => {
    dispatch('setErrors', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: payload.dataSetIdent,
      row: payload.row,
      oprIdent: payload.oprIdent,
      errors: payload.errors,
    })
  },
  addMessage: async ({ dispatch }, payload) => {
    dispatch('setErrors', payload.messages)
    await payload.messages.forEach(async message => {
      await dispatch('processCommands', {
        commands: message.commands,
      })
    })
  },
  processCommands: async ({ dispatch }, payload) => {
    const row = payload.row
    const where = payload.where
    const router = payload.router
    const window = payload.window
    const res = payload.res
    if (payload.commands) {
      await payload.commands.reduce(async (acc, item) => {
        await acc
        const command = item.command
        let payload = item.payload
        payload = {
          ...payload,
          row: row,
          where: where,
          router: router,
          window: window,
          res: res,
        }
        if (command) await dispatch(command, payload)
      }, Promise.resolve())
    }
  },
  hideError: ({ commit }, payload) => {
    commit('HIDE_ERROR', payload)
  },
  hideAllErrors: ({ state, dispatch }) => {
    state.errors.forEach(error => {
      dispatch('hideError', { error: error })
    })
  },
  clearHiddenErrors: ({ state, dispatch }) => {
    state.errors.forEach(error => {
      if (error.isVisible === false) {
        dispatch('removeError', { error: error })
      }
    })
  },
  removeError: ({ commit }, payload) => {
    commit('DEL_ERROR', payload)
  },
  clearErrors: ({ state, dispatch }) => {
    while (state.errors.length > 0) {
      dispatch('removeError', { error: state.errors[0] })
    }
  },
  removeMessage: ({ commit }, message) => {
    commit('DEL_MESSAGE', message)
  },
  clearMessages: ({ state, dispatch }) => {
    while (state.messages.length > 0) {
      dispatch('removeMessage', { message: state.messages[0] })
    }
  },
  setDataSetRefReqAll: ({ commit, getters }, payload) => {
    const appWindow = getters.getAppWindow(payload.appWindowIdent)
    const refreshAggregates = payload.refreshAggregates
    commit('SET_DATASET_REFREQ_ALL', { appWindow, refreshAggregates })
  },
  setDataSetRefReq: ({ commit, getters }, payload) => {
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    const { refreshAggregates, refCondition } = payload
    commit('SET_DATASET_REFREQ', { dataSet, refreshAggregates })
    if (getters.isDev && refCondition) commit('ADD_REFCONDITION', { dataSet, refCondition })
  },
  setDataSetCurrentRowIndex: ({ dispatch, commit }, payload) => {
    const dataSet = payload.dataSet
    const currentRowIndex = payload.currentRowIndex
    commit('SET_DATASET_CURRENT_ROW_INDEX', { dataSet, currentRowIndex })
    dispatch('refreshDataSetDependence', {
      appWindowIdent: payload.appWindowIdent,
      parentDataSetIdent: dataSet.dataSetIdent,
      refreshAggregates: 1,
    })
  },
  setDataSetCurrentRow: ({ dispatch, commit }, payload) => {
    const dataSet = payload.dataSet
    const row = payload.row
    let currentRowIndex = -1
    if (dataSet && dataSet.rows) {
      currentRowIndex = dataSet.rows.indexOf(row)
    }
    commit('SET_DATASET_CURRENT_ROW_INDEX', { dataSet, currentRowIndex })
    dispatch('refreshDataSetDependence', {
      appWindowIdent: payload.appWindowIdent,
      parentDataSetIdent: dataSet.dataSetIdent,
      refreshAggregates: 1,
    })
  },
  setDataSetCurrentRowFromFilter: ({ getters, commit, dispatch }, payload) => {
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    if (!dataSet || !payload.filter) return
    if (
      dataSet.state === dataSetUtils.state.added ||
      dataSet.state === dataSetUtils.state.progress
    )
      commit('SET_DATASET_CURRENT_ROW_FILTER', {
        dataSet: dataSet,
        filter: payload.filter,
      })
    else {
      const row = getters.appWindowDataSetFindRow(
        payload.appWindowIdent,
        payload.dataSetIdent,
        payload.filter
      )
      if (row) dispatch('setDataSetCurrentRow', { dataSet, row })
    }
  },
  setDataSetCurrentPivotIndex: async ({ commit, dispatch, getters }, payload) => {
    const dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    const pivotCurrentRowIndex = payload.pivotCurrentRowIndex
    const pivotCurrentColIndex = payload.pivotCurrentColIndex
    const pivotCurrentDataFieldIdent = payload.pivotCurrentDataFieldIdent
    if (
      dataSet &&
      dataSet.pivotCurrentRowIndex === pivotCurrentRowIndex &&
      dataSet.pivotCurrentColIndex === pivotCurrentColIndex &&
      dataSet.pivotCurrentDataFieldIdent === pivotCurrentDataFieldIdent
    )
      return
    commit('SET_DATASET_CURRENT_PIVOT_INDEX', {
      dataSet,
      pivotCurrentRowIndex,
      pivotCurrentColIndex,
      pivotCurrentDataFieldIdent,
    })
    await dispatch('refreshDataSetDependence', {
      appWindowIdent: payload.appWindowIdent,
      parentDataSetIdent: payload.dataSetIdent,
      refreshAggregates: 1,
    })
  },
  setDataSetColFieldAttr: async ({ getters, commit }, payload) => {
    const dataSet = payload.dataSet
    const colFieldIndex = payload.colFieldIndex
    const colFields = getters.getDataSetColFields(dataSet)
    const colField = colFields[colFieldIndex]
    const attr = payload.attr
    commit('SET_DATASET_COL_FIELD_ATTR', { dataSet, colField, attr })
  },
  setDataSetVisibleColFieldAttr: async ({ getters, commit }, payload) => {
    const dataSet = payload.dataSet
    const colFieldIndex = payload.colFieldIndex
    const colFields = getters.getDataSetVisibleColFields(dataSet)
    const colField = colFields[colFieldIndex]
    const attr = payload.attr
    commit('SET_DATASET_COL_FIELD_ATTR', { dataSet, colField, attr })
  },
  setDataSetFirstRow: ({ commit }, dataSet) => {
    if (dataSet != null && dataSet.rows.length > 1) {
      const currentRowIndex = 0
      commit('SET_DATASET_CURRENT_ROW_INDEX', { dataSet, currentRowIndex })
    }
  },
  setDataSetPrevRow: ({ commit }, dataSet) => {
    let currentRowIndex = dataSet.currentRowIndex
    if (currentRowIndex - 1 >= 0) {
      currentRowIndex--
      commit('SET_DATASET_CURRENT_ROW_INDEX', { dataSet, currentRowIndex })
    }
  },
  setDataSetPrevRowByStep: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    let currentRowIndex = dataSet.currentRowIndex
    let pageNo = dataSet.pageNo
    const step = payload.step || 1
    const pagesCount = getters.getDataSetPagesCount(dataSet)
    if (currentRowIndex - 1 >= 0) {
      currentRowIndex--
      commit('SET_DATASET_CURRENT_ROW_INDEX', { dataSet, currentRowIndex })
    } else if (currentRowIndex - 1 < 0 && pageNo > 1) {
      pageNo--
      dispatch('setDataSetPageNo', { dataSet, pageNo })
      currentRowIndex = dataSet.rows.length - step
      commit('SET_DATASET_CURRENT_ROW_INDEX', { dataSet, currentRowIndex })
    } else if (pageNo === 1 && pagesCount > 1) {
      pageNo = pagesCount
      dispatch('setDataSetPageNo', { dataSet, pageNo })
      currentRowIndex = dataSet.rows.length - step
      commit('SET_DATASET_CURRENT_ROW_INDEX', { dataSet, currentRowIndex })
    } else if (pageNo === 1) {
      currentRowIndex = dataSet.rows.length - step
      commit('SET_DATASET_CURRENT_ROW_INDEX', { dataSet, currentRowIndex })
    }
  },
  setDataSetNextRow: ({ commit }, dataSet) => {
    let currentRowIndex = dataSet.currentRowIndex
    if (currentRowIndex + 1 < dataSet.rows.length) {
      currentRowIndex++
      commit('SET_DATASET_CURRENT_ROW_INDEX', { dataSet, currentRowIndex })
    }
  },
  setDataSetNextRowByStep: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    let currentRowIndex = dataSet.currentRowIndex
    let pageNo = dataSet.pageNo
    const step = payload.step || 1
    const pagesCount = getters.getDataSetPagesCount(dataSet)
    if (currentRowIndex + step < dataSet.rows.length) {
      currentRowIndex++
      commit('SET_DATASET_CURRENT_ROW_INDEX', { dataSet, currentRowIndex })
    } else if (currentRowIndex + step >= dataSet.rows.length && pageNo < pagesCount) {
      pageNo++
      dispatch('setDataSetPageNo', { dataSet, pageNo })
    } else if (pageNo === pagesCount && pagesCount === 1) {
      currentRowIndex = 0
      commit('SET_DATASET_CURRENT_ROW_INDEX', { dataSet, currentRowIndex })
    } else if (pageNo === pagesCount) dispatch('setDataSetPageNoFirst', payload)
  },
  setDataSetFieldValue: async ({ dispatch, getters }, payload) => {
    if (payload.dataFieldIdent === null) return null
    if (
      payload.dataSet == null &&
      payload.appWindowIdent != null &&
      payload.dataSetIdent != null
    )
      payload.dataSet = getters.getDataSet(payload.appWindowIdent, payload.dataSetIdent)
    let action = 'setDataSetRowsFieldValue'
    switch (payload.sourceKind) {
      case 'aggregates':
        action = 'setDataSetAggregatesFieldValue'
        break
      case 'where':
        action = 'setDataSetWhereFieldValue'
        break
      case 'pivotValues':
        action = 'setDataSetPivotValuesFieldValue'
        break
    }
    await dispatch(action, payload)
  },
  setDataSetRowStatus: async ({ commit }, payload) => {
    commit('SET_DATASET_ROW_STATUS', {
      row: payload.row,
      status: payload.status,
    })
  },
  setDataSetRowsFieldValue: async ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const row = payload.row || getters.getDataSetCurrentRow(dataSet)
    if (!row) return
    let dataFieldValue = payload.dataFieldValue
    if (dataFieldValue === '') dataFieldValue = null
    const dataFieldIdent = payload.dataFieldIdent
    commit('SET_DATASET_ROWS_FIELD_VALUE', {
      row,
      dataFieldIdent,
      dataFieldValue,
    })
    await dispatch('setDataSetRowStatus', {
      row: row,
      status: dataSetUtils.rowState.modified,
    })
    const onCFActionIdent = getters.getFieldsOnCFActionIdentsOnCFActionIdent(
      dataSet,
      dataFieldIdent
    )
    if (onCFActionIdent && payload.onCFLock !== 1 && payload.oncfTimeout) {
      const action = getters.getDataSetAction(dataSet, onCFActionIdent)
      const timer = setTimeout(() => {
        dispatch('invokeAction', {
          appWindowIdent: dataSet.appWindowIdent,
          dataSetIdent: dataSet.dataSetIdent,
          actionIdent: onCFActionIdent,
          onCFDataFieldIdent: dataFieldIdent,
          row: row,
        })
      }, payload.oncfTimeout)
      commit('SET_DATASET_ACTION_TIMER', {
        row,
        action,
        timer,
      })
    } else if (onCFActionIdent && payload.onCFLock !== 1) {
      await dispatch('invokeAction', {
        appWindowIdent: dataSet.appWindowIdent,
        dataSetIdent: dataSet.dataSetIdent,
        actionIdent: onCFActionIdent,
        onCFDataFieldIdent: dataFieldIdent,
        row: row,
      })
    }
  },
  setDataSetAggregatesFieldValue: ({ commit }, payload) => {
    const dataSet = payload.dataSet
    const dataFieldValue = payload.dataFieldValue
    const dataFieldIdent = payload.dataFieldIdent
    commit('SET_DATASET_AGGREGATES_FIELD_VALUE', {
      dataSet,
      dataFieldIdent,
      dataFieldValue,
    })
  },
  setDataSetPivotValuesFieldValue: async ({ dispatch, commit, getters }, payload) => {
    const dataSet = payload.dataSet
    const dataFieldValue = payload.dataFieldValue
    const dataFieldIdent = payload.dataFieldIdent
    const pivotRow = payload.pivotRow
    const pivotCol = payload.pivotCol
    const pivotRowKey = payload.pivotRowKey
    const row =
      payload.dataSet.pivotValues[pivotRowKey] ||
      getters.getPivotEmptyRow(pivotCol, pivotRow)
    const colFields = getters.getDataSetColFields(dataSet)
    const rowFields = getters.getDataSetRowFields(dataSet)
    const rowTransformed = new constructors.csRow()
    colFields.forEach((item, index) => {
      rowTransformed[item.dataFieldIdent] = payload.dataSet.pivotCols[row[2]['c']][index][0]
    })
    rowFields.forEach((item, index) => {
      rowTransformed[item.dataFieldIdent] = payload.dataSet.pivotRows[row[2]['r']][index][0]
    })
    rowTransformed[payload.dataFieldIdent] = payload.dataFieldValue
    commit('SET_DATASET_PIVOT_VALUES_FIELD_VALUE', {
      dataSet,
      row,
      pivotRowKey,
      dataFieldIdent,
      dataFieldValue,
    })
    await dispatch('invokeAction', {
      appWindowIdent: payload.appWindowIdent,
      dataSetIdent: payload.dataSet.dataSetIdent,
      actionIdent: payload.dataSet.pivotUpdateActionIdent,
      row: rowTransformed,
    })
  },
  updateUrlQuery: async ({ dispatch, commit, getters }, payload) => {
    const { dataSet, selectedRowsAdd = [], selectedRowsDel = [] } = payload
    if (dataSet == null) return
    const { whereFieldsIdents, where = {} } = dataSet
    if (!await dispatch('checkRouter')) return
    if (whereFieldsIdents == null) return
    const query = JSON.parse(JSON.stringify(storeUtils.router.currentRoute.query))
    let changed = false
    whereFieldsIdents.forEach(whereFieldIdent => {
      if (whereFieldIdent.linkedParamName == null || whereFieldIdent.updateUrl === 0) return
      const whereVal = where[whereFieldIdent.dataFieldIdent]
      const queryVal = query[whereFieldIdent.linkedParamName]
      if (whereVal == queryVal) return
      if (whereVal == null || whereVal == whereFieldIdent.dataFieldValueDef) {
        if (query[whereFieldIdent.linkedParamName] !== undefined) {
          delete query[whereFieldIdent.linkedParamName]
          changed = true
        }
      } else {
        query[whereFieldIdent.linkedParamName] = whereVal
        changed = true
      }
    })
    selectedRowsAdd.forEach(row => {
      if (row._urlParamName == null) return
      switch (row._urlParamKind) {
        case 'list': {
          if (row._urlParamValue == null) break
          if (!(query[row._urlParamName] instanceof Array)) {
            if (query[row._urlParamName] != null)
              query[row._urlParamName] = [query[row._urlParamName]]
            else query[row._urlParamName] = []
          }
          const paramArr = query[row._urlParamName]
          const param = paramArr.find(p => p == row._urlParamValue)
          if (!param) {
            paramArr.push(row._urlParamValue)
            changed = true
          }
          break
        }
      }
    })
    selectedRowsDel.forEach(row => {
      if (row._urlParamName == null) return
      switch (row._urlParamKind) {
        case 'list':
          if (row._urlParamValue == null) break
          if (query[row._urlParamName] instanceof Array) {
            const paramArr = query[row._urlParamName]
            const idx = paramArr.findIndex(p => p == row._urlParamValue)
            paramArr.splice(idx, 1)
            changed = true
          } else if (query[row._urlParamName] == row._urlParamValue) {
            delete query[row._urlParamName]
            changed = true
          }
      }
    })
    if (changed) {
      storeUtils.pRouter = storeUtils.pRouter
        .then(() => {
          commit('LOCK_URL')
          return storeUtils.router.replace({ query })
        })
        .catch(e => {
          if (getters.isDev) {
            /* eslint-disable no-console */
            console.log(e)
            /* eslint-enable no-console */
          }
          commit('UNLOCK_URL')
        })
    }
  },
  setDataSetAttribFromWhere: async ({ commit, getters }, payload) => {
    const dataSet = payload.dataSet
    if (!dataSet) throw 'Empty dataSet'
    const dataFieldIdent = payload.dataFieldIdent
    const dataFieldValue = payload.dataFieldValue
    const whereFieldsIdent = getters.getDataSetWhereFieldsIdentsDataField(
      dataSet,
      dataFieldIdent
    )
    if (!whereFieldsIdent) return
    switch (whereFieldsIdent.dataSetAttrib) {
      case 'pageSizeIdent': {
        const pageSize = getters.getDataSetPageSizes(dataSet, dataFieldValue)
        if (pageSize == null) {
          break
        }
        commit('SET_DATASET_PAGESIZESIDENT', {
          dataSet: dataSet,
          pageSizesIdent: dataFieldValue,
          pageSize: pageSize.pageSize,
        })
        break
      }
      case 'pageNo': {
        const pagesCount = getters.getDataSetPagesCount(dataSet)
        if (pagesCount > 0 && dataFieldValue > pagesCount) break
        if (dataFieldValue < 1) break
        commit('SET_DATASET_PAGENOREF', {
          dataSet: dataSet,
          pageNo: dataFieldValue,
          pagesCount: pagesCount,
        })
        break
      }
      case 'sortIdent': {
        const layout = getters.getDataSetLayout(
          dataSet,
          getters.getDataSetLayoutIdent(dataSet)
        )
        if (getters.getDataSetSort(dataSet, dataFieldValue))
          commit('SET_LAYOUT_SORTIDENT', {
            layout: layout,
            sortIdent: dataFieldValue,
          })
        break
      }
      default:
        break
    }
  },
  setDataSetWhereFromAttrib: ({ getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const wfis = getters.getDataSetWhereFieldsIdents(dataSet)
    const attribName = payload.attribName
    if (wfis == null || attribName == null) return
    const wfi = wfis.find(f => f.dataSetAttrib === attribName)
    if (wfi == null) return
    dispatch('setDataSetWhereFieldValue', {
      dataSet: dataSet,
      dataFieldValue: payload.attribValue,
      dataFieldIdent: wfi.dataFieldIdent,
      attribLock: 1,
    })
  },
  setDataSetWhereFieldValue: async ({ dispatch, commit, getters }, payload) => {
    const dataSet = payload.dataSet
    const dataFieldIdent = payload.dataFieldIdent
    const attribLock = payload.attribLock
    const whereFieldsIdent = getters.getDataSetWhereFieldsIdentsDataField(
      dataSet,
      dataFieldIdent
    )
    if (!whereFieldsIdent) {
      /* eslint-disable no-console */
      console.error(
        'Brak parametru wejściowego [' +
        dataFieldIdent +
        '] dataSet [' +
        (dataSet ? dataSet.dataSetIdent : undefined) +
        ']'
      )
      /* eslint-enable no-console */
      return
    }
    const noRefReq =
      payload.noRefReq != null
        ? payload.noRefReq
        : whereFieldsIdent.refreshDependencyKind === 'none'
          ? 1
          : 0
    const dataFieldValue = payload.dataFieldValue =
      payload.dataFieldValue == null
        ? payload.dataFieldValue
        : dataSetUtils.checkDataType(
          payload.dataFieldValue,
          whereFieldsIdent ? whereFieldsIdent.formatType : 'string'
        )
    if (dataSet == null || dataFieldIdent == null) return
    if (attribLock !== 1) {
      await dispatch('setDataSetAttribFromWhere', payload)
    }
    commit('SET_DATASET_WHERE_FIELD_VALUE', {
      dataSet,
      dataFieldValue,
      dataFieldIdent,
      noRefReq,
    })
    /*
      this action is dispatched without appWindowIdent in payload
      if is called from setAppWindowData -> fillDatSetWhere -> setDataSetFieldValue
    */
    if (payload.appWindowIdent == undefined) return
    const onCFActionIdent = getters.getDataSetWhereOnCFActionIdentsOnCFActionIdent(
      dataSet,
      dataFieldIdent
    )
    if (onCFActionIdent && payload.onCFLock !== 1 && payload.oncfTimeout) {
      const action = getters.getDataSetAction(dataSet, onCFActionIdent)
      const timer = setTimeout(async () => {
        await dispatch('invokeAction', {
          appWindowIdent: payload.appWindowIdent,
          dataSetIdent: dataSet.dataSetIdent,
          actionIdent: onCFActionIdent,
          onCFDataFieldIdent: dataFieldIdent,
        })
        //
        if (payload.refreshLock !== 1) {
          if (!whereFieldsIdent || whereFieldsIdent.refreshDependency !== 1) return
          //
          await dispatch('refreshDataSet', {
            appWindowIdent: payload.appWindowIdent,
            dataSetIdent: dataSet.dataSetIdent,
            refreshAggregates: 1,
          })
        }
      }, payload.oncfTimeout)
      commit('SET_DATASET_ACTION_TIMER', {
        action,
        timer,
      })
    } else if (onCFActionIdent && payload.onCFLock !== 1) {
      await dispatch('invokeAction', {
        appWindowIdent: payload.appWindowIdent,
        dataSetIdent: dataSet.dataSetIdent,
        actionIdent: onCFActionIdent,
        onCFDataFieldIdent: dataFieldIdent,
      })
      //
      if (payload.refreshLock !== 1) {
        const whereFieldsIdent = getters.getDataSetWhereFieldsIdentsDataField(
          dataSet,
          dataFieldIdent
        )
        if (!whereFieldsIdent || whereFieldsIdent.refreshDependency !== 1) return
        //
        await dispatch('refreshDataSet', {
          appWindowIdent: payload.appWindowIdent,
          dataSetIdent: dataSet.dataSetIdent,
          refreshAggregates: 1,
        })
      }
    } else {
      if (payload.refreshLock !== 1) {
        const whereFieldsIdent = getters.getDataSetWhereFieldsIdentsDataField(
          dataSet,
          dataFieldIdent
        )
        if (!whereFieldsIdent || whereFieldsIdent.refreshDependency !== 1) return
        //
        await dispatch('refreshDataSet', {
          appWindowIdent: payload.appWindowIdent,
          dataSetIdent: dataSet.dataSetIdent,
          refreshAggregates: 1,
        })
      }
    }
  },
  setDataSetWhereListsFieldValue: ({ commit }, payload) => {
    const dataSet = payload.dataSet
    const lookupDataFieldIdent = payload.lookupDataFieldIdent
    const whereListValue = payload.whereListValue
    commit('SET_DATASET_WHERE_LISTS_FIELD_VALUE', {
      dataSet,
      lookupDataFieldIdent,
      whereListValue,
    })
  },
  setRowListsFieldValue: ({ commit, getters }, payload) => {
    const { dataSet, lookupDataFieldIdent, whereListValue } = payload
    const row = payload.row || getters.getDataSetCurrentRow(dataSet)
    commit('SET_ROW_LISTS_FIELD_VALUE', {
      row,
      lookupDataFieldIdent,
      whereListValue,
    })
  },
  pushRegisterSubscription: async ({ dispatch, getters, state, rootGetters }, payload) => {
    const dataToPost = {
      sessionId: getters.getSessionId(state),
      subscription: payload.subscription,
    }
    const response = await axios.post(
      rootGetters['csServerSettings/getAPIDataMethodURL'](
        'pushRegisterSubscription',
        payload.serverSide
      ),
      dataToPost,
      getters.getAPIDataRequestConfig()
    )
    await dispatch('setErrors', {
      errors: response.errors,
    })
  },
  pushSendTestMessage: async ({ dispatch, getters, state, rootGetters }, payload) => {
    const dataToPost = {
      sessionId: getters.getSessionId(state),
      subscription: payload.subscription,
      title: payload.title,
      url: payload.url,
      body: payload.body,
    }
    const response = await axios.post(
      rootGetters['csServerSettings/getAPIDataMethodURL'](
        'pushSendNotify',
        payload.serverSide
      ),
      dataToPost,
      getters.getAPIDataRequestConfig()
    )
    await dispatch('setErrors', {
      errors: response.errors,
    })
  },
  putFile: async ({ commit, rootGetters, dispatch }, payload) => {
    const file = payload.file
    const dataToPost = new FormData()
    dataToPost.append(payload.file.__uuid, file.file)
    const config = {
      onUploadProgress: function (progressEvent) {
        const progress = Math.round(progressEvent.loaded * 100 / progressEvent.total)
        commit('SET_FILE_PROGRESS', { file, progress })
      },
      headers: {
        'Content-Type': 'multipart/form-data',
      },
    }
    commit('SET_FILE_STATUS', {
      file: file,
      status: dataSetUtils.fileState.uploading,
    })
    const url = rootGetters['csServerSettings/getAPIDefinitionMethodURL'](
      'putFiles',
      payload.serverSide
    )
    const response = await axios.put(url, dataToPost, config)
    await dispatch('setErrors', {
      errors: response.errors,
    })
    if (response.data)
      await dispatch('setErrors', {
        errors: response.data.errors,
      })
    if (response.errors || response.data && response.data.errors)
      commit('SET_FILE_STATUS', {
        file: file,
        status: dataSetUtils.fileState.error,
      })
    else if (response.data) {
      commit('SET_FILE_STATUS', {
        file: file,
        status: dataSetUtils.fileState.uploaded,
      })
      commit('SET_FILE_DATA', {
        file: file,
        data: response.data.data ? response.data.data[0] : undefined,
      })
    }
  },
  putAllFiles: async ({ dispatch }, payload) => {
    if (!payload.dataSet) return
    const files = payload.dataSet.files
    await Promise.all(
      files.map(async file => {
        if (file._status !== dataSetUtils.fileState.uploaded)
          await dispatch('putFile', { file })
      })
    )
  },
  deleteAllFiles: async ({ commit }, payload) => {
    if (!payload.dataSet) return
    commit('DELETE_ALL_FILES', payload)
  },
  setScrollDirection: ({ commit }, payload) => {
    const dataSet = payload.dataSet
    const scrollDirection = payload.scrollDirection
    commit('SET_DATASET_SCROLL_DIRECTION', { dataSet, scrollDirection })
  },
  setColFieldOrd: async (
    { commit, getters, dispatch },
    { dataSet, colField, colFieldNew, setCurrent }
  ) => {
    let layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    if (!layout || !(layout.colFields instanceof Array)) return
    const idxOld = layout.colFields.indexOf(colField)
    const idxNew = layout.colFields.indexOf(colFieldNew)
    if (idxOld * idxNew < 0) return
    if (!layout.usrDef) layout = await dispatch('addUsrLayout', { layout, dataSet })
    commit('MOVE_COL_FIELD', {
      colFields: layout.colFields,
      idxOld: idxOld,
      idxNew: idxNew,
    })
    if (setCurrent)
      commit('SET_CURRENT_COL_FIELD_INDEX', {
        layout: layout,
        currentColFieldIndex: idxNew,
      })
  },
  setColFieldLockedLeftOrd: async (
    { commit, getters, dispatch },
    { dataSet, colField, colFieldNew, setCurrent }
  ) => {
    let layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    if (!layout || !(layout.colFields instanceof Array)) return
    const idxOld = layout.colFields.indexOf(colField)
    const idxNew = layout.colFields.indexOf(colFieldNew)
    if (idxOld * idxNew < 0) return
    if (!layout.usrDef) layout = await dispatch('addUsrLayout', { layout, dataSet })
    commit('MOVE_COL_FIELD', {
      colFields: layout.colFields,
      idxOld: idxOld,
      idxNew: idxNew,
    })
    if (setCurrent)
      commit('SET_CURRENT_COL_FIELD_LOCKED_LEFT_INDEX', {
        layout: layout,
        currentColFieldLockedLeftIndex: idxNew,
      })
  },
  setColFieldLockedRightOrd: async (
    { commit, getters, dispatch },
    { dataSet, colField, colFieldNew, setCurrent }
  ) => {
    let layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    if (!layout || !(layout.colFields instanceof Array)) return
    const idxOld = layout.colFields.indexOf(colField)
    const idxNew = layout.colFields.indexOf(colFieldNew)
    if (idxOld * idxNew < 0) return
    if (!layout.usrDef) layout = await dispatch('addUsrLayout', { layout, dataSet })
    commit('MOVE_COL_FIELD', {
      colFields: layout.colFields,
      idxOld: idxOld,
      idxNew: idxNew,
    })
    if (setCurrent)
      commit('SET_CURRENT_COL_FIELD_LOCKED_RIGHT_INDEX', {
        layout: layout,
        currentColFieldLockedRightIndex: idxNew,
      })
  },
  setSortColFieldOrd: async (
    { commit, getters, dispatch },
    { dataSet, sortColField, sortColFieldNew, setCurrent }
  ) => {
    let layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    let sortColFields = getters.getDataSetSortColFields(dataSet)
    if (!layout || !(sortColFields instanceof Array)) return
    const idxOld = sortColFields.indexOf(sortColField)
    const idxNew = sortColFields.indexOf(sortColFieldNew)
    if (idxOld * idxNew < 0) return
    if (!layout.usrDef) {
      layout = await dispatch('addUsrLayout', { layout, dataSet })
      sortColFields = getters.getDataSetSortColFields(dataSet)
    }
    commit('MOVE_SORT_COL_FIELD', { sortColFields, idxOld, idxNew })
    if (setCurrent) {
      const sort = getters.getDataSetSort(dataSet, getters.getDataSetSortIdent(dataSet))
      commit('SET_CURRENT_SORT_COL_FIELD_INDEX', {
        sort: sort,
        currentSortColFieldIndex: idxNew,
      })
    }
  },
  setColFieldVisibility: async (
    { commit, getters, dispatch },
    { colField, isVisible, layout, dataSet }
  ) => {
    if (!layout)
      layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    if (!layout) return
    if (layout.usrDef) {
      commit('SET_COL_FIELD_VISIBILITY', { colField, isVisible })
      return
    }
    const newL = await dispatch('addUsrLayout', { layout, dataSet })
    if (!newL) return
    const newCF = newL.colFields.find(c => c.dataFieldIdent === colField.dataFieldIdent)
    commit('SET_COL_FIELD_VISIBILITY', {
      colField: newCF,
      isVisible: isVisible,
    })
  },
  addUsrLayout: ({ commit, getters }, { layout, dataSet }) => {
    if (!dataSet) return null
    const appWindow = getters.getAppWindow(dataSet.appWindowIdent)
    if (!appWindow) return null
    const newL = layout ? JSON.parse(JSON.stringify(layout)) : constructors.csLayout()
    newL.usrDef = 1
    newL.device = getters.deviceType
    newL.owner = 1
    newL.global = 0
    commit('ADD_USR_LAYOUT', {
      layout: newL,
      dataSetUsrSettings: getters.getDataSetUsrSettings(appWindow, dataSet.dataSetIdent),
    })
    return newL
  },
  removeUsrLayout: ({ commit, getters }, { layoutIdent, dataSetIdent, appWindowIdent }) => {
    const appWindow = getters.getAppWindow(appWindowIdent)
    const dataSet = getters.getDataSet(appWindowIdent, dataSetIdent)
    const dataSetUsrSettings = getters.getDataSetUsrSettings(appWindow, dataSetIdent)
    if (!appWindow || !dataSet || !dataSetUsrSettings) return
    commit('REMOVE_USR_LAYOUT', { dataSetUsrSettings, layoutIdent })
    let layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    if (layout) return
    const layouts = getters.getDataSetLayoutIdents(dataSet)
    if (!(layouts instanceof Array)) return
    layout = layouts.find(l => l.default === 1)
    if (!layout) layout = layouts[0]
    commit('SET_LAYOUT_IDENT', {
      dataSet: dataSet,
      layoutIdent: layout.layoutIdent,
    })
  },
  setLayoutAttr({ commit }, payload) {
    commit('SET_LAYOUT_ATTR', payload)
  },
  setLayoutSortIdentToUsrDefinedWithCopy: ({ commit, getters, dispatch }, payload) => {
    const dataSet = payload.dataSet
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    const sortIdents = getters.getDataSetSortIdents(dataSet)
    const currentSort = getters.getDataSetSort(
      dataSet,
      getters.getDataSetSortIdent(dataSet)
    )
    if (layout === null || sortIdents === null) return
    const usrSort = sortIdents.find(item => {
      return item.isUsrDefined === 1
    })
    if (usrSort) {
      if (currentSort) {
        const sortColFields = JSON.parse(JSON.stringify(currentSort.sortColFields))
        commit('SET_SORT_IDENT_SORT_COL_FIELDS', {
          sort: usrSort,
          sortColFields: sortColFields,
        })
      }
      commit('SET_LAYOUT_SORTIDENT', {
        layout: layout,
        sortIdent: usrSort.sortIdent,
      })
      dispatch('setDataSetWhereFromAttrib', {
        dataSet: dataSet,
        attribName: 'sortIdent',
        attribValue: usrSort.sortIdent,
      })
      return usrSort
    }
  },
  setSortIdentSortColFields: ({ commit }, payload) => {
    commit('SET_SORT_IDENT_SORT_COL_FIELDS', { payload })
  },
  addSortColFieldToSortIdent: async (
    { commit, getters, dispatch },
    { dataSet, colField }
  ) => {
    if (!dataSet) return
    let layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    if (!layout.usrDef) layout = await dispatch('addUsrLayout', { layout, dataSet })
    let sort = getters.getDataSetSort(dataSet, getters.getDataSetSortIdent(dataSet))
    if (!sort.isUsrDefined)
      sort = await dispatch('setLayoutSortIdentToUsrDefinedWithCopy', {
        dataSet,
      })
    if (colField && sort) {
      const sortColField = {
        dataFieldIdent: colField.dataFieldIdent,
        dataFieldColTranslatedDesc: colField.dataFieldColTranslatedDesc,
        isDesc: 0,
        sortType: 0,
        ord: 0,
      }
      commit('ADD_SORT_IDENT_SORT_COL_FIELD', { sort, sortColField })
      const sortColFields = sort.sortColFields
      const currentSortColFieldIndex = sortColFields.length - 1
      commit('SET_CURRENT_SORT_COL_FIELD_INDEX', {
        sort,
        currentSortColFieldIndex,
      })
    }
  },
  deleteSortColField: async ({ commit, getters, dispatch }, { dataSet, sortColField }) => {
    if (!dataSet) return
    let layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    if (!layout.usrDef) layout = await dispatch('addUsrLayout', { layout, dataSet })
    let sort = getters.getDataSetSort(dataSet, getters.getDataSetSortIdent(dataSet))
    const index = sort.sortColFields.findIndex(
      s => s.dataFieldIdent === sortColField.dataFieldIdent
    )
    if (!sort.isUsrDefined)
      sort = await dispatch('setLayoutSortIdentToUsrDefinedWithCopy', {
        dataSet,
      })
    const ord = getters.getDataSetSortColFields(dataSet).length
    commit('DELETE_SORT_IDENT_SORT_COL_FIELD', {
      sort,
      index,
    })
    let currentSortColFieldIndex = getters.getCurrentSortColFieldIndex(dataSet)
    currentSortColFieldIndex =
      currentSortColFieldIndex >= ord - 1
        ? currentSortColFieldIndex === 0
          ? 0
          : currentSortColFieldIndex - 1
        : currentSortColFieldIndex
    commit('SET_CURRENT_SORT_COL_FIELD_INDEX', {
      sort,
      currentSortColFieldIndex,
    })
  },
  deleteAllSortColFields: async ({ commit, getters, dispatch }, { dataSet }) => {
    if (!dataSet) return
    let layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    if (!layout.usrDef) layout = await dispatch('addUsrLayout', { layout, dataSet })
    let sort = getters.getDataSetSort(dataSet, getters.getDataSetSortIdent(dataSet))
    if (!sort.isUsrDefined)
      sort = await dispatch('setLayoutSortIdentToUsrDefinedWithCopy', {
        dataSet,
      })
    commit('DELETE_ALL_SORT_IDENT_SORT_COL_FIELDS', { sort })
  },
  setSortColFieldsIsDesc: async (
    { getters, dispatch, commit },
    { dataSet, sortColField, isDesc }
  ) => {
    if (!dataSet) return
    let cloned = false
    let layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    if (!layout.usrDef) {
      layout = await dispatch('addUsrLayout', { layout, dataSet })
      cloned = true
    }
    let sort = getters.getDataSetSort(dataSet, getters.getDataSetSortIdent(dataSet))
    if (!sort.isUsrDefined) {
      sort = await dispatch('setLayoutSortIdentToUsrDefinedWithCopy', {
        dataSet,
      })
      cloned = true
    }
    if (cloned)
      sortColField = sort.sortColFields.find(
        s => s.dataFieldIdent === sortColField.dataFieldIdent
      )
    commit('SET_SORT_COL_FIELD_IS_DESC', { sortColField, isDesc })
  },
  setCurrentColField: ({ commit, getters }, payload) => {
    const dataSet = payload.dataSet
    if (dataSet == null) return
    const colField = payload.colField
    let currentColFieldIndex = -1
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    if (layout !== null && layout.colFields !== null) {
      if (colField) currentColFieldIndex = layout.colFields.indexOf(colField)
      else currentColFieldIndex = layout.colFields.findIndex(f => f.placement === 'default')
    }
    if (currentColFieldIndex > -1)
      commit('SET_CURRENT_COL_FIELD_INDEX', { layout, currentColFieldIndex })
  },
  setCurrentColFieldIndex: ({ commit, getters }, { dataSet, currentColFieldIndex }) => {
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    if (!layout || !layout.colFields) return
    commit('SET_CURRENT_COL_FIELD_INDEX', { layout, currentColFieldIndex })
  },

  setCurrentColFieldLockedLeft: ({ commit, getters }, payload) => {
    const dataSet = payload.dataSet
    if (dataSet == null) return
    const colField = payload.colField
    let currentColFieldLockedLeftIndex = -1
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    if (layout !== null && layout.colFields !== null) {
      if (colField) currentColFieldLockedLeftIndex = layout.colFields.indexOf(colField)
      else
        currentColFieldLockedLeftIndex = layout.colFields.findIndex(
          f => f.placement === 'left'
        )
    }
    if (currentColFieldLockedLeftIndex > -1)
      commit('SET_CURRENT_COL_FIELD_LOCKED_LEFT_INDEX', {
        layout,
        currentColFieldLockedLeftIndex,
      })
  },
  setCurrentColFieldLockedRight: ({ commit, getters }, payload) => {
    const dataSet = payload.dataSet
    if (dataSet == null) return
    const colField = payload.colField
    let currentColFieldLockedRightIndex = -1
    const layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    if (layout !== null && layout.colFields !== null) {
      if (colField) currentColFieldLockedRightIndex = layout.colFields.indexOf(colField)
      else
        currentColFieldLockedRightIndex = layout.colFields.findIndex(
          f => f.placement === 'right'
        )
    }
    if (currentColFieldLockedRightIndex > -1)
      commit('SET_CURRENT_COL_FIELD_LOCKED_RIGHT_INDEX', {
        layout,
        currentColFieldLockedRightIndex,
      })
  },

  setCurrentSortColField: ({ commit, getters }, payload) => {
    const dataSet = payload.dataSet
    if (dataSet == null) return
    const sortColField = payload.sortColField
    let currentSortColFieldIndex = -1
    const sort = getters.getDataSetSort(dataSet, getters.getDataSetSortIdent(dataSet))
    if (sort !== null && sort.sortColFields !== null) {
      currentSortColFieldIndex = sort.sortColFields.indexOf(sortColField)
    }
    if (currentSortColFieldIndex > -1)
      commit('SET_CURRENT_SORT_COL_FIELD_INDEX', {
        sort,
        currentSortColFieldIndex,
      })
  },
  setRefreshParams: ({ commit }, payload) => {
    commit('SET_REFRESH_PARAMS', payload)
  },
  deleteAppWindowsAll: async ({ state, getters, dispatch, commit }) => {
    commit('SETACTIVE_APPWINDOW', { appWindowIdent: null })
    commit('CLEAR_PATHS')
    dispatch('clearErrors', { state: state, dispatch: dispatch })
    dispatch('clearMessages', { state: state, dispatch: dispatch })
    for (const key in getters.csAppWindows) {
      commit('DELETE_APPWINDOW', { appWindowIdent: key })
    }
  },
  deleteModalWindow: async ({ commit, getters, dispatch }, payload) => {
    const modalWindows = getters.getAppWindowModalWindows(payload.appWindowIdent)
    const modalWindow =
      modalWindows && payload.modalWindowIndex
        ? modalWindows[payload.modalWindowIndex]
        : null
    if (!modalWindow) return
    const dataSet = getters.getDataSet(modalWindow.appWindowIdent, modalWindow.dataSetIdent)
    if (modalWindow.addNewRow) {
      commit('DELETE_ROW', {
        dataSet: dataSet,
        row: modalWindow.row,
      })
    } else if (payload.rollback) {
      dispatch('restoreRowBackup', {
        dataSet: dataSet,
        row: modalWindow.row,
      })
    }
    commit('CLEAR_ERRORFIELDS', {
      dataSet: dataSet,
    })
    commit('DELETE_MODAL_WINDOW', {
      modalWindows: modalWindows,
      modalWindowIndex: payload.modalWindowIndex,
    })
  },
  setDataSetAggrFieldCalc4Cols: ({ commit }, payload) => {
    commit('SET_DATASET_AGGRFIELD_CALC4COLS', payload)
  },
  setDataSetAggrFieldsCalc4Cols: ({ dispatch, getters }, payload) => {
    if (!payload.dataSet) return
    const aggrFields = getters.getDataSetAggrFields(payload.dataSet)
    aggrFields.forEach(aggrField => {
      dispatch('setDataSetAggrFieldCalc4Cols', {
        aggrField: aggrField,
        calc4Cols: payload.calc4Cols,
      })
    })
  },
  toggleDataSetAggrField: async (
    { commit, getters, dispatch },
    { dataSet, colField, aggrType }
  ) => {
    let layout = getters.getDataSetLayout(dataSet, getters.getDataSetLayoutIdent(dataSet))
    if (!layout) return
    if (!layout.usrDef) {
      layout = await dispatch('addUsrLayout', { layout, dataSet })
    }
    if (!layout) return
    const aggrFields = getters.getDataSetAggrFields(dataSet)
    const aggrField = aggrFields.find(f => f.dataFieldIdent === colField.dataFieldIdent)
    if (aggrField) {
      if (aggrField.aggrType === aggrType) {
        commit('REMOVE_DATA_SET_AGGR_FIELD', { aggrFields, aggrField })
      } else {
        commit('CHANGE_DATA_SET_AGGR_FIELD_AGGR_TYPE', { aggrField, aggrType })
      }
    } else {
      commit('ADD_DATA_SET_AGGR_FIELD', { aggrFields, colField, aggrType })
    }
  },
  setDataSetAggrFieldCalc4Rows: ({ commit }, payload) => {
    commit('SET_DATASET_AGGRFIELD_CALC4ROWS', payload)
  },
  setDataSetAggrFieldsCalc4Rows: ({ dispatch, getters }, payload) => {
    if (!payload.dataSet) return
    const aggrFields = getters.getDataSetAggrFields(payload.dataSet)
    aggrFields.forEach(aggrField => {
      dispatch('setDataSetAggrFieldCalc4Rows', {
        aggrField: aggrField,
        calc4Rows: payload.calc4Rows,
      })
    })
  },
  setAutoFocus: async () => {
    if (document) {
      const c = document.querySelector('[autofocus="autofocus"]')
      const a = document.activeElement
      if (c) {
        if (a !== c) {
          c.focus()
          c.click()
        }
      }
    }
  },
  selectedRowsRepair: ({ getters }) => {
    for (const appWindowIdent in getters.csAppWindows) {
      const appWindow = getters.getAppWindow(appWindowIdent)
      for (const dataSetIdent in appWindow.dataSets) {
        const dataSet = getters.getDataSet(appWindowIdent, dataSetIdent)
        if (dataSet.selectedRows != null && dataSet.rows != null) {
          for (let i = 0; i < dataSet.selectedRows.length; i++) {
            const row = dataSet.rows.find(
              rowTmp => dataSet.selectedRows[i].__uuid == rowTmp.__uuid
            )
            if (row != null && dataSet.selectedRows[i] != row) {
              dataSet.selectedRows[i] = row
            }
          }
        }
      }
    }
  },
  setActiveMetaInfo: ({ commit, getters }, { path }) => {
    if (!path) return

    const pathMetaInfo = getters.pathMetaInfo(path)
    const activeMetaInfo = getters.activeMetaInfo

    if (activeMetaInfo != pathMetaInfo)
      commit('SET_ACTIVE_META_INFO', { path, metaInfo: pathMetaInfo })
  },
  setPathMetaInfo: async ({ commit, dispatch }, { path, metaInfo }) => {
    if (!path) return

    commit('SET_PATH_META_INFO', { path, metaInfo })

    if (storeUtils.router.currentRoute.path === path)
      await dispatch('setActiveMetaInfo', { path })
  },
  activeMetaInfoRepair: ({ getters, dispatch }) => {
    const path = getters.activeMetaInfoPath
    dispatch('setActiveMetaInfo', { path })
  },
  repairStateAfterReplaceState: ({ dispatch }) => {
    dispatch('selectedRowsRepair')
    dispatch('activeMetaInfoRepair')
  },
  moveAppWindowTab: (
    { commit },
    { appWindowUniqIdentToMove, appWindowUniqIdentToReference, direction }
  ) => {
    if (direction === 0) return
    commit('MOVE_APP_WINDOW_TAB', {
      appWindowUniqIdentToMove,
      appWindowUniqIdentToReference,
      direction,
    })
  },
  setWhereFieldsIdentsDataFieldIsActive: (
    { commit },
    { whereFieldsIdentsDataField, isActive }
  ) => {
    commit('SET_WHERE_FIELDS_IDENTS_DATA_FIELD_IS_ACTIVE', {
      whereFieldsIdentsDataField,
      isActive,
    })
  },
  clearErrorField: ({ commit }, { errorFields, dataFieldIdent }) => {
    commit('CLEAR_ERRORFIELD', { errorFields, dataFieldIdent })
  },
}
